@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2006-2015 Yves Renard, Julien Pommier.
+ Copyright (C) 2006-2017 Yves Renard, Julien Pommier.
 
  This file is a part of GetFEM++
 
@@ -18,11 +18,9 @@
  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
 ===========================================================================*/
-// $Id: gf_slice_get.cc 5051 2015-07-23 21:01:32Z logari81 $
+// $Id$
 #include <map>
-#include <getfemint_misc.h>
-#include <getfemint_mesh.h>
-#include <getfemint_mesh_slice.h>
+#include <getfemint_workspace.h>
 #include <getfem/getfem_mesh_slice.h>
 #include <getfem/getfem_export.h>
 
@@ -136,11 +134,10 @@ struct sub_gf_slice_get : virtual public dal::static_stored_object {
   int arg_in_min, arg_in_max, arg_out_min, arg_out_max;
   virtual void run(getfemint::mexargs_in& in,
 		   getfemint::mexargs_out& out,
-		   getfemint_mesh_slice *mi_sl,
-		   const getfem::stored_mesh_slice *sl) = 0;
+		   getfem::stored_mesh_slice *sl) = 0;
 };
 
-typedef boost::intrusive_ptr<sub_gf_slice_get> psub_command;
+typedef std::shared_ptr<sub_gf_slice_get> psub_command;
 
 // Function to avoid warning in macro with unused arguments.
 template <typename T> static inline void dummy_func(T &) {}
@@ -149,16 +146,14 @@ template <typename T> static inline void dummy_func(T &) {}
     struct subc : public sub_gf_slice_get {				\
       virtual void run(getfemint::mexargs_in& in,			\
 		       getfemint::mexargs_out& out,			\
-		       getfemint_mesh_slice *mi_sl,			\
-		       const getfem::stored_mesh_slice *sl)		\
-      { dummy_func(in); dummy_func(out); dummy_func(mi_sl);		\
-	dummy_func(sl); code }						\
+		       getfem::stored_mesh_slice *sl)			\
+      { dummy_func(in); dummy_func(out); dummy_func(sl); code }		\
     };									\
-    psub_command psubc = new subc;					\
+    psub_command psubc = std::make_shared<subc>();			\
     psubc->arg_in_min = arginmin; psubc->arg_in_max = arginmax;		\
     psubc->arg_out_min = argoutmin; psubc->arg_out_max = argoutmax;	\
     subc_tab[cmd_normalize(name)] = psubc;				\
-  }                           
+  }
 
 
 
@@ -291,7 +286,7 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
        slicer.push_back_action(action); slicer.exec(*sl);
 
        /* return a point list, a connectivity array, and optionnaly a list of edges with are part of the slice */
-       double nan = get_NaN();
+       double nan = ::nan("");
        dal::bit_vector bv = m.points().index();
        darray P = out.pop().create_darray(m.dim(), unsigned(bv.last_true()+1));
        iarray T1 = out.pop().create_iarray(2, unsigned(m.nb_convex() - slice_edges.card()));
@@ -339,7 +334,17 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
       Return the mesh on which the slice was taken.@*/
     sub_command
       ("linked mesh", 0, 0, 0, 1,
-       out.pop().from_object_id(mi_sl->linked_mesh_id(), MESH_CLASS_ID);
+       id_type id = workspace().object((const void *)(&(sl->linked_mesh())));
+       if (id == id_type(-1)) {
+	 auto pst = workspace().hidden_object(workspace().object(sl),
+					      &sl->linked_mesh());
+	 if (!pst.get()) THROW_INTERNAL_ERROR;
+	 std::shared_ptr<getfem::mesh> pm = 
+	   std::const_pointer_cast<getfem::mesh>
+	   (std::dynamic_pointer_cast<const getfem::mesh>(pst));
+	 id = store_mesh_object(pm);
+       }
+       out.pop().from_object_id(id, MESH_CLASS_ID);
        );
 
     /*@GET m = ('mesh')
@@ -347,7 +352,9 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
       (identical to 'linked mesh')@*/
     sub_command
       ("mesh", 0, 0, 0, 1,
-       out.pop().from_object_id(mi_sl->linked_mesh_id(), MESH_CLASS_ID);
+       id_type id = workspace().object((const void *)(&(sl->linked_mesh())));
+       if (id == id_type(-1)) THROW_INTERNAL_ERROR;
+       out.pop().from_object_id(id, MESH_CLASS_ID);
        );
 
 
@@ -412,8 +419,8 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
        int count = 1;
        if (in.remaining()) {
 	 do {
-	   if (in.remaining() >= 2 && in.front().is_mesh_fem()) {
-	     const getfem::mesh_fem &mf = *in.pop().to_const_mesh_fem();
+	   if (in.remaining() >= 2 && is_meshfem_object(in.front())) {
+	     const getfem::mesh_fem &mf = *to_meshfem_object(in.pop());
 	     
 	     darray U = in.pop().to_darray();
 	     in.last_popped().check_trailing_dimension(int(mf.nb_dof()));
@@ -488,8 +495,8 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
        if (edges) exp.exporting_mesh_edges();
        if (in.remaining()) {
 	 do {
-	   if (in.remaining() >= 2 && in.front().is_mesh_fem()) {
-	     const getfem::mesh_fem &mf = *in.pop().to_const_mesh_fem();
+	   if (in.remaining() >= 2 && is_meshfem_object(in.front())) {
+	     const getfem::mesh_fem &mf = *to_meshfem_object(in.pop());
 	     
 	     darray U = in.pop().to_darray();
 	     in.last_popped().check_trailing_dimension(int(mf.nb_dof()));
@@ -525,8 +532,8 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
 	 name = in.pop().to_string();
        exp.write(*sl,name);
        while (in.remaining()) {
-	 if (in.remaining() >= 3 && in.front().is_mesh_fem()) {
-	   const getfem::mesh_fem *mf = in.pop().to_const_mesh_fem();
+	 if (in.remaining() >= 3 && is_meshfem_object(in.front())) {
+	     const getfem::mesh_fem *mf = to_meshfem_object(in.pop());
 	   
 	   darray U = in.pop().to_darray();
 	   in.last_popped().check_trailing_dimension(int(mf->nb_dof()));
@@ -577,8 +584,7 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
  
   if (m_in.narg() < 2)  THROW_BADARG( "Wrong number of input arguments");
 
-  getfemint_mesh_slice *mi_sl = m_in.pop().to_getfemint_mesh_slice();
-  const getfem::stored_mesh_slice *sl = &mi_sl->mesh_slice();
+  getfem::stored_mesh_slice *sl = to_slice_object(m_in.pop());
   std::string init_cmd   = m_in.pop().to_string();
   std::string cmd        = cmd_normalize(init_cmd);
 
@@ -588,7 +594,7 @@ void gf_slice_get(getfemint::mexargs_in& m_in,
     check_cmd(cmd, it->first.c_str(), m_in, m_out, it->second->arg_in_min,
 	      it->second->arg_in_max, it->second->arg_out_min,
 	      it->second->arg_out_max);
-    it->second->run(m_in, m_out, mi_sl, sl);
+    it->second->run(m_in, m_out, sl);
   }
   else bad_cmd(init_cmd);
 
