@@ -1,4 +1,4 @@
-% Copyright (C) 2009-2016 Yves Renard.
+% Copyright (C) 2009-2017 Yves Renard.
 %
 % This file is a part of GetFEM++
 %
@@ -364,7 +364,7 @@ end
 % Solve the problem
 if (~solved)
   gf_model_get(md, 'test tangent matrix', 1e-6, 10, 0.1);
-  gf_model_get(md, 'solve', 'max_res', 1E-9, 'very noisy', 'max_iter', niter); % ,  'lsearch', 'simplest'); % , 'with pseudo potential');
+  gf_model_get(md, 'solve', 'max_res', 1E-9, 'very noisy', 'max_iter', niter); % ,  'lsearch', 'simplest');
 end;
 
 U = gf_model_get(md, 'variable', 'u');
