@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # Python GetFEM++ interface
 #
-# Copyright (C) 2009-2015 Luis Saavedra, Yves Renard.
+# Copyright (C) 2009-2017 Luis Saavedra, Yves Renard.
 #
 # This file is a part of GetFEM++
 #
@@ -24,7 +24,7 @@
   This program is used to check that python-getfem is working. This is
   also a good example of use of GetFEM++.
 
-  $Id: demo_crack.py 5102 2015-10-18 18:20:28Z logari81 $
+  $Id$
 """
 try:
   import getfem as gf
@@ -33,7 +33,7 @@ except ImportError:
   sys.path.append('../../src/python/')
   import getfem as gf
 else:
-  print "module getfem not found!"
+  print("module getfem not found!")
 
 import numpy as np
 import math
@@ -187,5 +187,5 @@ VM = md.compute_isotropic_linearized_Von_Mises_or_Tresca('u_cut', 'lambda', 'mu'
 
 mfv.export_to_pos('crack.pos', V, 'V', Ve, 'Ve', mfvm, VM, 'Von Mises')
 
-print 'You can view the solution with (for example):'
-print 'gmsh crack.pos'
+print('You can view the solution with (for example):')
+print('gmsh crack.pos')
