@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2000-2016 Yves Renard
+ Copyright (C) 2000-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -28,15 +28,270 @@
 
 namespace bgeot {
 
+  std::vector<scalar_type>& __aux1(){
+    DEFINE_STATIC_THREAD_LOCAL(std::vector<scalar_type>, v);
+    return v;
+  }
+  
+  std::vector<scalar_type>& __aux2(){
+    DEFINE_STATIC_THREAD_LOCAL(std::vector<scalar_type>, v);
+    return v;
+  }
+  
+  std::vector<scalar_type>& __aux3(){
+    DEFINE_STATIC_THREAD_LOCAL(std::vector<scalar_type>, v);
+    return v;
+  }
+  
+  std::vector<int>& __ipvt_aux(){
+    DEFINE_STATIC_THREAD_LOCAL(std::vector<int>, vi);
+    return vi;
+  }
+  
+  // Optimized matrix mult for small matrices. To be verified.
+  // Multiply the matrix A of size MxN by B of size NxP in C of size MxP
+  void mat_mult(const scalar_type *A, const scalar_type *B, scalar_type *C,
+		size_type M, size_type N, size_type P) {
+    if (N != 0) {
+      auto itC = C; auto itB = B;
+      for (size_type j = 0; j < P; ++j, itB += N)
+	for (size_type i = 0; i < M; ++i, ++itC) {
+	  auto itA = A+i, itB1 = itB;
+	  *itC = (*itA) * (*itB1);
+	  for (size_type k = 1; k < N; ++k)
+	    { itA += M; ++itB1; *itC += (*itA) * (*itB1); }
+	}
+    } else std::fill(C, C+M*P, scalar_type(0));
+  }
+
+  // Optimized matrix mult for small matrices.
+  // Multiply the matrix A of size MxN by the transpose of B of size PxN
+  // in C of size MxP
+  void mat_tmult(const scalar_type *A, const scalar_type *B, scalar_type *C,
+		 size_type M, size_type N, size_type P) {
+    auto itC = C;
+    switch (N) {
+    case 0 : std::fill(C, C+M*P, scalar_type(0)); break;
+    case 1 : 
+      for (size_type j = 0; j < P; ++j)
+	for (size_type i = 0; i < M; ++i, ++itC) {
+	  auto itA = A+i, itB = B+j;
+	  *itC = (*itA) * (*itB);
+	}
+      break;
+    case 2 :
+      for (size_type j = 0; j < P; ++j)
+	for (size_type i = 0; i < M; ++i, ++itC) {
+	  auto itA = A+i, itB = B+j;
+	  *itC = (*itA) * (*itB);
+	  itA += M; itB += P; *itC += (*itA) * (*itB);
+	}
+      break;
+    case 3 :
+      for (size_type j = 0; j < P; ++j)
+	for (size_type i = 0; i < M; ++i, ++itC) {
+	  auto itA = A+i, itB = B+j;
+	  *itC = (*itA) * (*itB);
+	  itA += M; itB += P; *itC += (*itA) * (*itB);
+	  itA += M; itB += P; *itC += (*itA) * (*itB);
+	}
+      break;
+    default :
+      for (size_type j = 0; j < P; ++j)
+	for (size_type i = 0; i < M; ++i, ++itC) {
+	  auto itA = A+i, itB = B+j;
+	  *itC = (*itA) * (*itB);
+	  for (size_type k = 1; k < N; ++k)
+	    { itA += M; itB += P; *itC += (*itA) * (*itB); }
+	}
+    }
+  }
+
+
+
+
+  // Optimized lu_factor for small square matrices
+  size_type lu_factor(scalar_type *A, std::vector<int> &ipvt,
+		      size_type N) {
+    size_type info(0), i, j, jp, N_1 = N-1;
+      
+    if (N) {
+      for (j = 0; j < N_1; ++j) {
+	auto it = A + (j*(N+1));
+	scalar_type max = gmm::abs(*it); jp = j;
+	for (i = j+1; i < N; ++i) {
+	  scalar_type ap = gmm::abs(*(++it));
+	  if (ap > max) { jp = i; max = ap; }
+	}
+	ipvt[j] = int(jp + 1);
+	
+	if (max == scalar_type(0)) { info = j + 1; break; }
+        if (jp != j) {
+	  auto it1 = A+jp, it2 = A+j;
+	  for (i = 0; i < N; ++i, it1+=N, it2+=N) std::swap(*it1, *it2);
+	}
+	it = A + (j*(N+1)); max = *it++;
+	for (i = j+1; i < N; ++i) *it++ /= max; 
+	auto it22 = A + (j*N + j+1), it11 = it22;
+	auto it3 = A + ((j+1)*N+j);
+	for (size_type l = j+1; l < N; ++l) {
+	  it11 += N;
+	  auto it1 = it11, it2 = it22;
+	  scalar_type a = *it3; it3 += N;
+	  for (size_type k = j+1; k < N; ++k) *it1++ -= *it2++ * a;
+	}
+
+      }
+      ipvt[N-1] = int(N);
+    }
+    return info;
+  }
+
+  static void lower_tri_solve(const scalar_type *T, scalar_type *x, int N,
+			      bool is_unit) {
+    scalar_type x_j;
+    for (int j = 0; j < N; ++j) {
+      auto itc = T + j*N, it = itc+(j+1), ite = itc+N;
+      auto itx = x + j;
+      if (!is_unit) *itx /= itc[j];
+      x_j = *itx++;
+      for (; it != ite ; ++it, ++itx) *itx -= x_j * (*it);
+    }
+  }
+
+  static void upper_tri_solve(const scalar_type *T, scalar_type *x, int N,
+			      bool is_unit) {
+    scalar_type x_j;
+    for (int j = N - 1; j >= 0; --j) {
+      auto itc = T + j*N, it = itc, ite = itc+j;
+      auto itx = x;
+      if (!is_unit) x[j] /= itc[j];
+      for (x_j = x[j]; it != ite ; ++it, ++itx) *itx -= x_j * (*it);
+    }
+  }
+
+  static void lu_solve(const scalar_type *LU, const std::vector<int> &ipvt, 
+		       scalar_type *x, scalar_type *b, int N) {
+    std::copy(b, b+N, x);
+    for(int i = 0; i < N; ++i)
+      { int perm = ipvt[i]-1; if(i != perm) std::swap(x[i], x[perm]); }
+    bgeot::lower_tri_solve(LU, x, N, true);
+    bgeot::upper_tri_solve(LU, x, N, false);
+  }
+
+  scalar_type lu_det(const scalar_type *LU, const std::vector<int> &ipvt,
+		     size_type N) {
+    scalar_type det(1);
+    for (size_type j = 0; j < N; ++j) det *= *(LU+j*(N+1));
+    for(int i = 0; i < int(N); ++i) if (i != ipvt[i]-1) { det = -det; }
+    return det;
+  }
+
+  scalar_type lu_det(const scalar_type *A, size_type N) {
+    switch (N) {
+    case 1: return *A;
+    case 2: return (*A) * (A[3]) - (A[1]) * (A[2]);
+    case 3: 
+      {
+	scalar_type a0 = A[4]*A[8] - A[5]*A[7], a3 = A[5]*A[6] - A[3]*A[8];
+	scalar_type a6 = A[3]*A[7] - A[4]*A[6];
+	return A[0] * a0 + A[1] * a3 + A[2] * a6;
+      }
+    default:
+      {
+	size_type NN = N*N;
+	if (__aux1().size() < NN) __aux1().resize(N*N);
+	std::copy(A, A+NN, __aux1().begin());
+	__ipvt_aux().resize(N);
+	lu_factor(&(*(__aux1().begin())), __ipvt_aux(), N);
+	return lu_det(&(*(__aux1().begin())), __ipvt_aux(), N);
+      }
+    }
+  }
+
+  void lu_inverse(const scalar_type *LU, const std::vector<int> &ipvt,
+		  scalar_type *A, size_type N) {
+    __aux2().resize(N); gmm::clear(__aux2());
+    __aux3().resize(N);
+    for(size_type i = 0; i < N; ++i) {
+      __aux2()[i] = scalar_type(1);
+      bgeot::lu_solve(LU, ipvt, A+i*N, &(*(__aux2().begin())), int(N));
+      __aux2()[i] = scalar_type(0);
+    }
+  }
+
+  scalar_type lu_inverse(scalar_type *A, size_type N, bool doassert) {
+    switch (N) {
+    case 1:
+      {
+	scalar_type det = *A;
+	GMM_ASSERT1(det != scalar_type(0), "Non invertible matrix");
+	*A = scalar_type(1)/det;
+	return det;
+      }
+    case 2:
+      {
+	scalar_type a = *A, b = A[2], c = A[1], d = A[3];
+	scalar_type det = a * d - b * c;
+	GMM_ASSERT1(det != scalar_type(0), "Non invertible matrix");
+	*A++ =  d/det;  *A++ /= -det; *A++ /= -det;  *A =  a/det;
+	return det;
+      }
+    case 3:
+      {
+	scalar_type a0 = A[4]*A[8] - A[5]*A[7], a1 = A[5]*A[6] - A[3]*A[8];
+	scalar_type a2 = A[3]*A[7] - A[4]*A[6];
+	scalar_type det =  A[0] * a0 + A[1] * a1 + A[2] * a2;
+	GMM_ASSERT1(det != scalar_type(0), "Non invertible matrix");
+	scalar_type a3 = (A[2]*A[7] - A[1]*A[8]), a6 = (A[1]*A[5] - A[2]*A[4]);
+	scalar_type a4 = (A[0]*A[8] - A[2]*A[6]), a7 = (A[2]*A[3] - A[0]*A[5]);
+	scalar_type a5 = (A[1]*A[6] - A[0]*A[7]), a8 = (A[0]*A[4] - A[1]*A[3]);
+	*A++ = a0 / det; *A++ = a3 / det; *A++ = a6 / det;
+	*A++ = a1 / det; *A++ = a4 / det; *A++ = a7 / det;
+	*A++ = a2 / det; *A++ = a5 / det; *A++ = a8 / det;
+	return det;
+      }
+    default:
+      {
+	size_type NN = N*N;
+	if (__aux1().size() < NN) __aux1().resize(NN);
+	std::copy(A, A+NN, __aux1().begin());
+	__ipvt_aux().resize(N);
+	size_type info = lu_factor(&(*(__aux1().begin())), __ipvt_aux(), N);
+	if (doassert) GMM_ASSERT1(!info, "Non invertible matrix, pivot = "
+				  << info);
+	if (!info) lu_inverse(&(*(__aux1().begin())), __ipvt_aux(), A, N);
+	return lu_det(&(*(__aux1().begin())), __ipvt_aux(), N);
+      }
+    }
+  }
+
+
+
   void geometric_trans::compute_K_matrix
-    (const base_matrix &G, const base_matrix &pc, base_matrix &K) const{
-    gmm::mult(G, pc, K);
+    (const base_matrix &G, const base_matrix &pc, base_matrix &K) const {
+    // gmm::mult(G, pc, K);
+    // Faster than the lapack call on my config ...
+    size_type N=gmm::mat_nrows(G), P=gmm::mat_nrows(pc), Q=gmm::mat_ncols(pc);
+    if (N && P && Q) {
+      auto itK = K.begin();
+      for (size_type j = 0; j < Q; ++j) {
+	auto itpc_j = pc.begin() + j*P, itG_b = G.begin();
+	for (size_type i = 0; i < N; ++i, ++itG_b) {
+	  auto itG = itG_b, itpc = itpc_j;
+	  register scalar_type a = *(itG) * (*itpc);
+	  for (size_type k = 1; k < P; ++k)
+	    { itG += N; a += *(itG) * (*++itpc); }
+	  *itK++ = a;
+	}
+      }
+    } else gmm::clear(K);
   }
 
   const base_node& geotrans_interpolation_context::xref() const {
     if (!have_xref()) {
       if (pspt_) xref_ = (*pspt_)[ii_];
-      else GMM_ASSERT1(false, "missing xref");
+      else GMM_ASSERT1(false, "Missing xref");
     }
     return xref_;
   }
@@ -51,56 +306,93 @@ namespace bgeot {
   }
 
   void geotrans_interpolation_context::compute_J(void) const {
-    GMM_ASSERT1(have_G() && have_pgt(), "unable to compute B\n");
+    GMM_ASSERT1(have_G() && have_pgt(), "Unable to compute J\n");
     size_type P = pgt_->structure()->dim();
-    base_matrix CS(P,P);
+    const base_matrix &KK = K();
     if (P != N()) {
-      gmm::mult(gmm::transposed(K()), K(), CS);
+      B_factors.base_resize(P, P);
+      gmm::mult(gmm::transposed(KK), KK, B_factors);
       // gmm::abs below because on flat convexes determinant could be -1e-27.
-      J_ = ::sqrt(gmm::abs(gmm::lu_det(CS)));
+      J__ = J_ =::sqrt(gmm::abs(bgeot::lu_inverse(&(*(B_factors.begin())), P)));
+    } else {
+      auto it = &(*(KK.begin()));
+      switch (P) {
+      case 1: J__ = *it; break;
+      case 2: J__ = (*it) * (it[3]) - (it[1]) * (it[2]); break;
+      case 3:
+	{
+	  B_.base_resize(P, P); // co-factors
+	  auto itB = B_.begin();
+	  scalar_type a0 = itB[0] = it[4]*it[8] - it[5]*it[7];
+	  scalar_type a1 = itB[1] = it[5]*it[6] - it[3]*it[8];
+	  scalar_type a2 = itB[2] = it[3]*it[7] - it[4]*it[6];
+	  J__ = it[0] * a0 + it[1] * a1 + it[2] * a2;
+	} break;
+      default:
+      	B_factors.base_resize(P, P); // store factorization for B computation
+      	gmm::copy(gmm::transposed(KK), B_factors);
+      	ipvt.resize(P);
+	bgeot::lu_factor(&(*(B_factors.begin())), ipvt, P);
+	J__ = bgeot::lu_det(&(*(B_factors.begin())), ipvt, P);
+	break;
+      }
+      J_ = gmm::abs(J__);
     }
-    else
-      J_ = gmm::abs(gmm::lu_det(K()));
+    have_J_ = true;
   }
 
   const base_matrix& geotrans_interpolation_context::K() const {
     if (!have_K()) {
-      GMM_ASSERT1(have_G() && have_pgt(), "unable to compute K\n");
+      GMM_ASSERT1(have_G() && have_pgt(), "Unable to compute K\n");
       size_type P = pgt_->structure()->dim();
-      K_.resize(N(), P);
+      K_.base_resize(N(), P);
       if (have_pgp()) {
-        GMM_ASSERT1(ii_ < pgp_->get_ppoint_tab()->size(),
-                    "Invalid index " << ii_ << " should be < "
-                    << pgp_->get_ppoint_tab()->size());
-
-        if (&pgp_->grad(ii_) == 0) { cerr << "OULA!! " << ii_ << "\n"; }
-        else if (pgp_->grad(ii_).size() == 0) { cerr << "OUCH\n"; }
-
-        pgt()->compute_K_matrix(G(), pgp_->grad(ii_), K_);
+	pgt_->compute_K_matrix(*G_, pgp_->grad(ii_), K_);
       } else {
-        base_matrix pc(pgt()->nb_points(), P);
-        pgt()->poly_vector_grad(xref(), pc);
-        pgt()->compute_K_matrix(G(), pc, K_);
+	PC.base_resize(pgt_->nb_points(), P);
+        pgt_->poly_vector_grad(xref(), PC);
+	pgt_->compute_K_matrix(*G_, PC, K_);
       }
+      have_K_ = true;
     }
     return K_;
   }
 
   const base_matrix& geotrans_interpolation_context::B() const {
     if (!have_B()) {
-      GMM_ASSERT1(have_G() && have_pgt(), "unable to compute B\n");
-      size_type P = pgt_->structure()->dim();
-      B_.resize(N(), P);
-      if (P != N()) {
-        base_matrix CS(P,P);
-        gmm::mult(gmm::transposed(K()), K(), CS);
-        // gmm::abs below because on flat convexes determinant could be -1e-27.
-        J_ = ::sqrt(gmm::abs(gmm::lu_inverse(CS)));
-        gmm::mult(K(), CS, B_);
+      const base_matrix &KK = K();
+      size_type P = pgt_->structure()->dim(), N_ = gmm::mat_nrows(KK);
+      B_.base_resize(N_, P);
+      if (!have_J_) compute_J();
+      GMM_ASSERT1(J__ != scalar_type(0), "Non invertible matrix");
+      if (P != N_) {
+        gmm::mult(KK, B_factors, B_);
       } else {
-        gmm::copy(gmm::transposed(K()), B_);
-        J_ = gmm::abs(gmm::lu_inverse(B_));
+	switch(P) {
+	case 1: B_(0, 0) = scalar_type(1) / J__;  break;
+	case 2:
+	  {
+	    auto it = &(*(KK.begin())); auto itB = &(*(B_.begin()));
+	    *itB++ = it[3] / J__; *itB++ = -it[2] / J__; 
+	    *itB++ = -it[1] / J__; *itB = (*it) / J__;
+	  } break;
+	case 3:
+	  {
+	    auto it = &(*(KK.begin())); auto itB = &(*(B_.begin()));
+	    *itB++ /= J__; *itB++ /= J__; *itB++ /= J__; 
+	    *itB++ = (it[2]*it[7] - it[1]*it[8]) / J__;
+	    *itB++ = (it[0]*it[8] - it[2]*it[6]) / J__;
+	    *itB++ = (it[1]*it[6] - it[0]*it[7]) / J__;
+	    *itB++ = (it[1]*it[5] - it[2]*it[4]) / J__;
+	    *itB++ = (it[2]*it[3] - it[0]*it[5]) / J__;
+	    *itB   = (it[0]*it[4] - it[1]*it[3]) / J__;
+	  } break;
+	default:
+	  bgeot::lu_inverse(&(*(B_factors.begin())), ipvt, &(*(B_.begin())), P);
+	  break;
+	}
       }
+      have_B_ = true;
     }
     return B_;
   }
@@ -109,12 +401,13 @@ namespace bgeot {
     if (!have_B3()) {
       const base_matrix &BB = B();
       size_type P=gmm::mat_ncols(BB), N_=gmm::mat_nrows(BB);
-      B3_.resize(N_*N_, P*P);
+      B3_.base_resize(N_*N_, P*P);
       for (short_type i = 0; i < P; ++i)
         for (short_type j = 0; j < P; ++j)
           for (short_type k = 0; k < N_; ++k)
             for (short_type l = 0; l < N_; ++l)
               B3_(k + N_*l, i + P*j) = BB(k, i) * BB(l, j);
+      have_B3_ = true;
     }
     return B3_;
   }
@@ -123,16 +416,16 @@ namespace bgeot {
     if (!have_B32()) {
       const base_matrix &BB = B();
       size_type P=gmm::mat_ncols(BB), N_=gmm::mat_nrows(BB);
-      B32_.resize(N_*N_, P);
+      B32_.base_resize(N_*N_, P);
       if (!pgt()->is_linear()) {
         base_matrix B2(P*P, P), Htau(N_, P*P);
         if (have_pgp()) {
           gmm::mult(G(), pgp_->hessian(ii_), Htau);
         } else {
           /* very inefficient of course... */
-          base_matrix pc(pgt()->nb_points(), P*P);
-          pgt()->poly_vector_hess(xref(), pc);
-          gmm::mult(G(), pc, Htau);
+	  PC.base_resize(pgt()->nb_points(), P*P);
+          pgt()->poly_vector_hess(xref(), PC);
+          gmm::mult(G(), PC, Htau);
         }
         for (short_type i = 0; i < P; ++i)
           for (short_type j = 0; j < P; ++j)
@@ -141,48 +434,18 @@ namespace bgeot {
                 B2(i + P*j, k) += Htau(l, i + P*j) * BB(l,k);
         gmm::mult(B3(), B2, B32_);
       } else gmm::clear(B32_);
+      have_B32_ = true;
     }
     return B32_;
   }
 
- void geotrans_interpolation_context::set_ii(size_type ii__) {
-    if (ii_ == ii__) return;
-    if (have_K() && !pgt()->is_linear()) { K_.resize(0,0); }
-    if (have_B() && !pgt()->is_linear()) { B_.resize(0,0); }
-    if (have_B3() && !pgt()->is_linear()) {
-      B3_.resize(0,0); B32_.resize(0,0); }
-    if (J_ >= scalar_type(0) && !pgt()->is_linear()) J_ = scalar_type(-1);
-    xref_.clear(); xreal_.clear();
-    ii_=ii__;
-  }
-
   void geotrans_interpolation_context::set_xref(const base_node& P) {
     xref_ = P;
-    if (have_K() && !pgt()->is_linear()) { K_.resize(0,0); }
-    if (have_B() && !pgt()->is_linear()) { B_.resize(0,0); }
-    if (have_B3() && !pgt()->is_linear()) {
-      B3_.resize(0,0); B32_.resize(0,0); }
-    if (J_ >= scalar_type(0) && !pgt()->is_linear()) J_ = scalar_type(-1);
-    xreal_.clear(); ii_ = size_type(-1); pspt_ = 0;
+    if (pgt_ && !pgt()->is_linear())
+      { have_K_ = have_B_ = have_B3_ = have_B32_ = have_J_ = false; }
+    xreal_.resize(0); ii_ = size_type(-1); pspt_ = 0;
   }
 
-  geotrans_interpolation_context::geotrans_interpolation_context() :
-    G_(0), pgt_(0), pgp_(0), pspt_(0), ii_(size_type(-1)), J_(-1) {}
-  geotrans_interpolation_context::geotrans_interpolation_context
-  (bgeot::pgeotrans_precomp pgp__, size_type ii__, const base_matrix& G__) :
-    G_(&G__), pgt_(pgp__->get_trans()), pgp_(pgp__),
-    pspt_(pgp__->get_ppoint_tab()), ii_(ii__), J_(-1) {}
-  geotrans_interpolation_context::geotrans_interpolation_context
-  (bgeot::pgeometric_trans pgt__, bgeot::pstored_point_tab pspt__,
-   size_type ii__,  const base_matrix& G__) :
-    G_(&G__), pgt_(pgt__), pgp_(0),
-    pspt_(pspt__), ii_(ii__), J_(-1) {}
-  geotrans_interpolation_context::geotrans_interpolation_context
-  (bgeot::pgeometric_trans pgt__, const base_node& xref__,
-   const base_matrix& G__) :
-    xref_(xref__), G_(&G__), pgt_(pgt__), pgp_(0), pspt_(0),
-    ii_(size_type(-1)), J_(-1) {}
-
 
   base_node geometric_trans::transform(const base_node &pt,
                                        const base_matrix &G) const {
@@ -198,17 +461,18 @@ namespace bgeot {
     return P;
   }
 
-  void geometric_trans::fill_standard_vertices(void) {
+  void geometric_trans::fill_standard_vertices() {
     vertices_.resize(0);
     for (size_type ip = 0; ip < nb_points(); ++ip) {
       bool vertex = true;
       for (size_type i = 0; i < cvr->points()[ip].size(); ++i)
         if (gmm::abs(cvr->points()[ip][i]) > 1e-10
-            && gmm::abs(cvr->points()[ip][i]-1.0) > 1e-10)
+            && gmm::abs(cvr->points()[ip][i]-1.0) > 1e-10
+	    && gmm::abs(cvr->points()[ip][i]+1.0) > 1e-10)
           { vertex = false; break; }
       if (vertex) vertices_.push_back(ip);
     }
-    assert(vertices_.size() >= dim());
+    assert(vertices_.size() > dim());
   }
 
   /* ******************************************************************** */
@@ -219,14 +483,43 @@ namespace bgeot {
   struct igeometric_trans : public geometric_trans {
 
     std::vector<FUNC> trans;
+    mutable std::vector<std::vector<FUNC>> grad_, hess_;
 
+    void compute_grad_() const {
+      size_type R = trans.size();
+      dim_type n = dim();
+      grad_.resize(R);
+      for (size_type i = 0; i < R; ++i) {
+	grad_[i].resize(n);
+	for (dim_type j = 0; j < n; ++j) {
+	  grad_[i][j] = trans[i]; grad_[i][j].derivative(j);
+	}
+      }
+    }
+
+    void compute_hess_() const {
+      size_type R = trans.size();
+      dim_type n = dim();
+      hess_.resize(R);
+      for (size_type i = 0; i < R; ++i) {
+	hess_[i].resize(n*n);
+	for (dim_type j = 0; j < n; ++j) {
+	  for (dim_type k = 0; k < n; ++k) {
+	    hess_[i][j+k*n] = trans[i];
+	    hess_[i][j+k*n].derivative(j); hess_[i][j+k*n].derivative(k);
+	  }
+	}
+      }
+    }
+    
     virtual void poly_vector_val(const base_node &pt, base_vector &val) const {
       val.resize(nb_points());
       for (size_type k = 0; k < nb_points(); ++k)
         val[k] = to_scalar(trans[k].eval(pt.begin()));
     }
 
-    virtual void poly_vector_val(const base_node &pt, const convex_ind_ct &ind_ct,
+    virtual void poly_vector_val(const base_node &pt,
+				 const convex_ind_ct &ind_ct,
                                  base_vector &val) const {
       size_type nb_funcs=ind_ct.size();
       val.resize(nb_funcs);
@@ -235,40 +528,35 @@ namespace bgeot {
     }
 
     virtual void poly_vector_grad(const base_node &pt, base_matrix &pc) const {
+      if (!(grad_.size())) compute_grad_();
       FUNC PP;
-      pc.resize(nb_points(),dim());
+      pc.base_resize(nb_points(),dim());
       for (size_type i = 0; i < nb_points(); ++i)
-        for (dim_type n = 0; n < dim(); ++n) {
-          PP = trans[i];
-          PP.derivative(n);
-          pc(i, n) = to_scalar(PP.eval(pt.begin()));
-        }
+        for (dim_type n = 0; n < dim(); ++n)
+          pc(i, n) = to_scalar(grad_[i][n].eval(pt.begin()));
     }
 
     virtual void poly_vector_grad(const base_node &pt,
 				  const convex_ind_ct &ind_ct,
                                   base_matrix &pc) const {
+      if (!(grad_.size())) compute_grad_();
       FUNC PP;
       size_type nb_funcs=ind_ct.size();
-      pc.resize(nb_funcs,dim());
+      pc.base_resize(nb_funcs,dim());
       for (size_type i = 0; i < nb_funcs; ++i)
-        for (dim_type n = 0; n < dim(); ++n) {
-          PP = trans[ind_ct[i]];
-          PP.derivative(n);
-          pc(i, n) = to_scalar(PP.eval(pt.begin()));
-        }
+        for (dim_type n = 0; n < dim(); ++n)
+          pc(i, n) = to_scalar(grad_[ind_ct[i]][n].eval(pt.begin()));
     }
 
     virtual void poly_vector_hess(const base_node &pt, base_matrix &pc) const {
+      if (!(hess_.size())) compute_hess_();
       FUNC PP, QP;
-      pc.resize(nb_points(),dim()*dim());
+      pc.base_resize(nb_points(),dim()*dim());
       for (size_type i = 0; i < nb_points(); ++i)
         for (dim_type n = 0; n < dim(); ++n) {
-          QP = trans[i]; QP.derivative(n);
-          for (dim_type m = 0; m <= n; ++m) {
-            PP = QP; PP.derivative(m);
-            pc(i, n*dim()+m) = pc(i, m*dim()+n) = to_scalar(PP.eval(pt.begin()));
-          }
+          for (dim_type m = 0; m <= n; ++m)
+            pc(i, n*dim()+m) = pc(i, m*dim()+n) =
+	      to_scalar(hess_[i][m*dim()+n].eval(pt.begin()));
         }
     }
 
@@ -276,6 +564,7 @@ namespace bgeot {
 
   typedef igeometric_trans<base_poly> poly_geometric_trans;
   typedef igeometric_trans<polynomial_composite> comppoly_geometric_trans;
+  typedef igeometric_trans<base_rational_fraction> fraction_geometric_trans;
 
   /* ******************************************************************** */
   /* transformation on simplex.                                           */
@@ -557,6 +846,78 @@ namespace bgeot {
     return geometric_trans_descriptor(name.str());
   }
 
+  /* ******************************************************************** */
+  /*	Pyramidal geometric transformation of order k=1 or 2.             */
+  /* ******************************************************************** */
+
+  struct pyramidal_trans_: public fraction_geometric_trans  {
+    pyramidal_trans_(short_type k) {
+      cvr = pyramidal_element_of_reference(k);
+      size_type R = cvr->structure()->nb_points();
+      is_lin = false;
+      complexity_ = k;
+      trans.resize(R);
+
+      if (k == 1) {
+	base_rational_fraction Q(read_base_poly(3, "x*y"),    // Q = xy/(1-z)
+				 read_base_poly(3, "1-z"));
+	trans[0] = (read_base_poly(3, "1-x-y-z") + Q)*0.25;
+	trans[1] = (read_base_poly(3, "1+x-y-z") - Q)*0.25;
+	trans[2] = (read_base_poly(3, "1-x+y-z") - Q)*0.25;
+	trans[3] = (read_base_poly(3, "1+x+y-z") + Q)*0.25;
+	trans[4] = read_base_poly(3, "z");
+      } else if (k == 2) {
+        base_poly xi0  = read_base_poly(3, "(1-z-x)*0.5");
+        base_poly xi1  = read_base_poly(3, "(1-z-y)*0.5");
+        base_poly xi2  = read_base_poly(3, "(1-z+x)*0.5");
+        base_poly xi3  = read_base_poly(3, "(1-z+y)*0.5");
+	base_poly x    = read_base_poly(3, "x");
+	base_poly y    = read_base_poly(3, "y");
+	base_poly z    = read_base_poly(3, "z");
+	base_poly ones = read_base_poly(3, "1");
+	base_poly un_z = read_base_poly(3, "1-z");
+	base_rational_fraction Q(read_base_poly(3, "1"), un_z); // Q = 1/(1-z)
+	trans[ 0] = Q*Q*xi0*xi1*(x*y-z*un_z);
+	trans[ 1] = Q*Q*xi0*xi1*xi2*(xi1*2.-un_z)*4.;
+	trans[ 2] = Q*Q*xi1*xi2*(-x*y-z*un_z);
+	trans[ 3] = Q*Q*xi3*xi0*xi1*(xi0*2.-un_z)*4.;
+	trans[ 4] = Q*Q*xi0*xi1*xi2*xi3*16.;
+	trans[ 5] = Q*Q*xi1*xi2*xi3*(xi2*2.-un_z)*4.;
+	trans[ 6] = Q*Q*xi3*xi0*(-x*y-z*un_z);
+	trans[ 7] = Q*Q*xi2*xi3*xi0*(xi3*2.-un_z)*4.;
+	trans[ 8] = Q*Q*xi2*xi3*(x*y-z*un_z);
+	trans[ 9] = Q*z*xi0*xi1*4.;
+	trans[10] = Q*z*xi1*xi2*4.;
+	trans[11] = Q*z*xi3*xi0*4.;
+	trans[12] = Q*z*xi2*xi3*4.;
+	trans[13] = read_base_poly(3, "z*(2*z-1)");
+      }
+      fill_standard_vertices();
+    }
+  };
+  
+  static pgeometric_trans
+    pyramidal_gt(gt_param_list& params,
+                     std::vector<dal::pstatic_stored_object> &dependencies) {
+    GMM_ASSERT1(params.size() == 1, "Bad number of parameters : "
+		<< params.size() << " should be 1.");
+    GMM_ASSERT1(params[0].type() == 0, "Bad type of parameters");
+    int k = int(::floor(params[0].num() + 0.01));
+    
+    dependencies.push_back(pyramidal_element_of_reference(dim_type(k)));
+    return std::make_shared<pyramidal_trans_>(dim_type(k));
+  }
+  
+  pgeometric_trans pyramidal_geotrans(short_type k) {
+    static short_type k_ = -1;
+    static pgeometric_trans pgt = 0;
+    if (k != k_) {
+      std::stringstream name;
+      name << "GT_PYRAMID(" << k << ")";
+      pgt = geometric_trans_descriptor(name.str());
+    }
+    return pgt;  
+  }
 
   /* ******************************************************************** */
   /*    Misc function.                                                    */
@@ -636,6 +997,7 @@ namespace bgeot {
       add_suffix("LINEAR_PRODUCT", linear_product_gt);
       add_suffix("LINEAR_QK", linear_qk);
       add_suffix("Q2_INCOMPLETE", Q2_incomplete_gt);
+      add_suffix("PYRAMID", pyramidal_gt);
     }
   };
 
