@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2000-2016 Yves Renard
+ Copyright (C) 2000-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -66,7 +66,7 @@ namespace bgeot
     if (P != N) {
       pgt->compute_K_matrix(G, pc, K);
       gmm::mult(gmm::transposed(K), K, CS);
-      gmm::lu_inverse(CS);
+      bgeot::lu_inverse(&(*(CS.begin())), P);
       gmm::mult(K, CS, B);
     }
     else {
@@ -75,8 +75,8 @@ namespace bgeot
       base_matrix KT(K.nrows(), K.ncols());
       pgt->compute_K_matrix(G, pc, KT);
       gmm::copy(gmm::transposed(KT), K);
-      gmm::copy(K,B);
-      gmm::lu_inverse(K); B.swap(K); 
+      gmm::copy(K, B);
+      bgeot::lu_inverse(&(*(K.begin())), P); B.swap(K); 
     }
   }
 
@@ -108,7 +108,6 @@ namespace bgeot
 
     //    for (size_type j = 0; j < pgt->nb_points(); ++j) 
     //       cout << "point " << j << " : " << cvpts[j] << endl;
-
     converged = true;
     base_node xn(P), y, z,x0;
     /* find an initial guess */
@@ -123,7 +122,6 @@ namespace bgeot
 
     base_node vres(P);
     base_node rn(xreal); rn -= y; 
-
     pgt->poly_vector_grad(x, pc);
     update_B();
     
