@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2002-2016 Yves Renard
+ Copyright (C) 2002-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -39,8 +39,8 @@ namespace bgeot {
     int ret = 0;
     for (; itx != itex; ++itx, ++ity) {
       long a = long(sfloor((*itx) * c1)), b = long(sfloor((*ity) * c1));
-      if ((gmm::abs(a) > scalar_type(base))
-          || (gmm::abs(b) > scalar_type(base))) { 
+      if ((scalar_type(gmm::abs(a)) > scalar_type(base))
+          || (scalar_type(gmm::abs(b)) > scalar_type(base))) { 
         exp_max++; c_max /= scalar_type(base);
         return (*this)(x,y);
       }
@@ -81,7 +81,7 @@ namespace bgeot {
       base_matrix pc(pgt->nb_points() , N);
       base_matrix B0(N, P);
 
-      vectors_to_base_matrix(G, m.points_of_convex(cv));
+      m.points_of_convex(cv, G);
 
       base_node x(N); gmm::clear(x);
       pgt->poly_vector_grad(x, pc);
