@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2000-2016 Yves Renard
+ Copyright (C) 2000-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -111,7 +111,7 @@ namespace bgeot {
     size_type complexity_; /* either the degree or the refinement of the
                             *  transformation */
 
-    void fill_standard_vertices(void);
+    void fill_standard_vertices();
   public :
 
     /// Dimension of the reference element.
@@ -221,6 +221,7 @@ namespace bgeot {
   pgeometric_trans APIDECL linear_product_geotrans(pgeometric_trans pg1,
                                            pgeometric_trans pg2);
   pgeometric_trans APIDECL Q2_incomplete_geotrans(dim_type nc);
+  pgeometric_trans APIDECL pyramidal_geotrans(short_type k);
 
   /**
      Get the geometric transformation from its string name.
@@ -394,24 +395,29 @@ namespace bgeot {
      are heavily described in the GetFEM++ Kernel Documentation.
   */
   class APIDECL geotrans_interpolation_context {
-    mutable base_node xref_; /** reference point */
+  protected:
+    mutable base_node xref_;  /** reference point */
     mutable base_node xreal_; /** transformed point */
-    const base_matrix *G_; /** pointer to the matrix of real nodes of the convex */
-    mutable base_matrix K_,B_, B3_, B32_; /** see documentation (getfem kernel doc) for more details */
+    const base_matrix *G_;    /** pointer to the matrix of real nodes of the convex */
+    mutable base_matrix K_, B_, B3_, B32_; /** see documentation (getfem kernel doc) for more details */
     pgeometric_trans pgt_;
     pgeotrans_precomp pgp_;
     pstored_point_tab pspt_; /** if pgp != 0, it is the same as pgp's one */
-    size_type ii_; /** index of current point in the pgp */
-    mutable scalar_type J_; /** Jacobian */
+    size_type ii_;           /** index of current point in the pgp */
+    mutable scalar_type J_, J__; /** Jacobian */
+    mutable base_matrix PC, B_factors;
+    mutable base_vector aux1, aux2;
+    mutable std::vector<int> ipvt;
+    mutable bool have_J_, have_B_, have_B3_, have_B32_, have_K_;
     void compute_J(void) const;
   public:
     bool have_xref() const { return !xref_.empty(); }
     bool have_xreal() const { return !xreal_.empty(); }
     bool have_G() const { return G_ != 0; }
-    bool have_K() const { return !K_.empty(); }
-    bool have_B() const { return !B_.empty(); }
-    bool have_B3() const { return !B3_.empty(); }
-    bool have_B32() const { return !B32_.empty(); }
+    bool have_K() const { return have_K_; }
+    bool have_B() const { return have_B_; }
+    bool have_B3() const { return have_B3_; }
+    bool have_B32() const { return have_B32_; }
     bool have_pgt() const { return pgt_ != 0; }
     bool have_pgp() const { return pgp_ != 0; }
     /// coordinates of the current point, in the reference convex.
@@ -427,27 +433,71 @@ namespace bgeot {
     /** matrix whose columns are the vertices of the convex */
     const base_matrix& G() const { return *G_; }
     /** get the Jacobian of the geometric trans (taken at point @c xref() ) */
-    scalar_type J() const { if (J_ < scalar_type(0)) compute_J(); return J_; }
+    scalar_type J() const { if (!have_J_) compute_J(); return J_; }
     size_type N() const { if (have_G()) return G().nrows();
       else if (have_xreal()) return xreal_.size();
       else GMM_ASSERT2(false, "cannot get N"); return 0; }
     size_type ii() const { return ii_; }
     bgeot::pgeotrans_precomp pgp() const { return pgp_; }
     /** change the current point (assuming a geotrans_precomp_ is used) */
-    void set_ii(size_type ii__);
+    void set_ii(size_type ii__) {
+      if (ii_ != ii__) {
+	if (pgt_ && !pgt()->is_linear())
+	  { have_K_ = have_B_ = have_B3_ = have_B32_ = have_J_ = false; }
+	xref_.resize(0); xreal_.resize(0);
+	ii_=ii__;
+      }
+    }
     /** change the current point (coordinates given in the reference convex) */
     void set_xref(const base_node& P);
-    geotrans_interpolation_context();
+    void change(bgeot::pgeotrans_precomp pgp__,
+		size_type ii__,
+		const base_matrix& G__) {
+      G_ = &G__; pgt_ = pgp__->get_trans(); pgp_ = pgp__;
+      pspt_ = pgp__->get_ppoint_tab(); ii_ = ii__;
+      have_J_ = have_B_ = have_B3_ = have_B32_ = have_K_ = false;
+      xref_.resize(0); xreal_.resize(0);
+    }
+    void change(bgeot::pgeometric_trans pgt__,
+		bgeot::pstored_point_tab pspt__,
+		size_type ii__,
+		const base_matrix& G__) {
+      G_ = &G__; pgt_ = pgt__; pgp_ = 0; pspt_ = pspt__; ii_ = ii__;
+      have_J_ = have_B_ = have_B3_ = have_B32_ = have_K_ = false;
+      xref_.resize(0); xreal_.resize(0);
+    }
+    void change(bgeot::pgeometric_trans pgt__,
+		const base_node& xref__,
+		const base_matrix& G__) {
+      xref_ = xref__; G_ = &G__; pgt_ = pgt__; pgp_ = 0; pspt_ = 0;
+      ii_ = size_type(-1);
+      have_J_ = have_B_ = have_B3_ = have_B32_ = have_K_ = false;
+      xreal_.resize(0);
+    }
+    
+    geotrans_interpolation_context()
+      : G_(0), pgt_(0), pgp_(0), pspt_(0), ii_(size_type(-1)),
+      have_J_(false), have_B_(false), have_B3_(false), have_B32_(false),
+      have_K_(false) {}
     geotrans_interpolation_context(bgeot::pgeotrans_precomp pgp__,
                                    size_type ii__,
-                                   const base_matrix& G__);
+                                   const base_matrix& G__)
+      : G_(&G__), pgt_(pgp__->get_trans()), pgp_(pgp__),
+      pspt_(pgp__->get_ppoint_tab()), ii_(ii__), have_J_(false), have_B_(false),
+      have_B3_(false), have_B32_(false), have_K_(false) {}
     geotrans_interpolation_context(bgeot::pgeometric_trans pgt__,
                                    bgeot::pstored_point_tab pspt__,
                                    size_type ii__,
-                                   const base_matrix& G__);
+                                   const base_matrix& G__)
+      : G_(&G__), pgt_(pgt__), pgp_(0),
+      pspt_(pspt__), ii_(ii__), have_J_(false), have_B_(false), have_B3_(false),
+      have_B32_(false), have_K_(false) {}
     geotrans_interpolation_context(bgeot::pgeometric_trans pgt__,
                                    const base_node& xref__,
-                                   const base_matrix& G__);
+                                   const base_matrix& G__)
+      : xref_(xref__), G_(&G__), pgt_(pgt__), pgp_(0), pspt_(0),
+      ii_(size_type(-1)),have_J_(false), have_B_(false), have_B3_(false),
+      have_B32_(false), have_K_(false)  {}
   };
 
   /* Function allowing the add of an geometric transformation method outwards
@@ -459,6 +509,23 @@ namespace bgeot {
   (std::string name, dal::naming_system<geometric_trans>::pfunction f);
 
 
+  /* Optimized operation for small matrices                               */
+  scalar_type lu_det(const scalar_type *A, size_type N);
+  scalar_type lu_inverse(scalar_type *A, size_type N, bool doassert = true);
+  inline scalar_type lu_det(const base_matrix &A)
+  { return lu_det(&(*(A.begin())), A.nrows()); }
+  inline scalar_type lu_inverse(base_matrix &A, bool doassert = true)
+  { return lu_inverse(&(*(A.begin())), A.nrows(), doassert); }
+  // Optimized matrix mult for small matrices.
+  // Multiply the matrix A of size MxN by B of size NxP in C of size MxP
+  void mat_mult(const scalar_type *A, const scalar_type *B, scalar_type *C,
+		size_type M, size_type N, size_type P);
+  // Optimized matrix mult for small matrices.
+  // Multiply the matrix A of size MxN by the transpose of B of size PxN
+  // in C of size MxP
+  void mat_tmult(const scalar_type *A, const scalar_type *B, scalar_type *C,
+		 size_type M, size_type N, size_type P);
+
 }  /* end of namespace bgeot.                                             */
 
 
