@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2000-2016 Julien Pommier
+ Copyright (C) 2000-2017 Julien Pommier
 
  This file is a part of GetFEM++
 
@@ -174,9 +174,9 @@ namespace bgeot {
     typedef const T * const_iterator;
 
     reference operator[](size_type l)
-    { GMM_ASSERT2(l < size(), "out of range"); return base()[l]; }
+    { GMM_ASSERT2(l <=size(), "out of range, l="<<l<<"size="<<size()); return base()[l]; }
     value_type operator[](size_type l) const
-    { GMM_ASSERT2(l < size(), "out of range"); return const_base()[l]; }
+    { GMM_ASSERT2(l <= size(), "out of range, l="<<l<<"size="<<size()); return const_base()[l]; }
     value_type at(size_type l) const { return const_base()[l]; }
     iterator begin() { return base(); }
     const_iterator begin() const { return const_base(); }
@@ -394,7 +394,7 @@ namespace bgeot {
   template <class VEC_CONT>
   void vectors_to_base_matrix(base_matrix &G, const VEC_CONT &a) {
     size_type P = (*(a.begin())).size(), NP = a.end() - a.begin();
-    G.resize(P, NP);
+    G.base_resize(P, NP);
     typename VEC_CONT::const_iterator it = a.begin(), ite = a.end();
     base_matrix::iterator itm = G.begin();
     for (; it != ite; ++it, itm += P)
