@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2002-2015 Yves Renard
+ Copyright (C) 2002-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -29,12 +29,12 @@
 
 ===========================================================================*/
 
-/** @file dal_static_stored_objects.h 
+/** @file dal_static_stored_objects.h
 @author  Yves Renard <Yves.Renard@insa-lyon.fr>
 @date February 19, 2005
 @brief Stores interdependent getfem objects.
 
-Stored object :  
+Stored object :
 
 A type of object to be stored should derive from
 dal::static_stored_object and a key should inherit from
@@ -43,9 +43,9 @@ static_stored_object_key with an overloaded "compare" method.
 To store a new object, you have to test if the object is not
 already stored and then call dal::add_stored_object:
 @code
-if (!search_stored_object(your_object_key(parameters))) {
-add_stored_object(new your_object_key(parameters),
-new your_object(parameters));
+pstatic_stored_object_key p = make_shared<your_object_key>(parameters);
+if (!search_stored_object(p)) {
+add_stored_object(p, make_shared<your_object>(parameters));
 }
 @endcode
 You can add a dependency of your new object with
@@ -57,12 +57,12 @@ and then your object will be automatically deleted if the second object is
 deleted.
 The dependency can be added within the add_stored_object call:
 @code
-add_stored_object(new your_object_key(parameters), 
+add_stored_object(new your_object_key(parameters),
 new your_object(parameters),
 dependency);
 @endcode
 
-Boost intrusive_ptr are used.
+std::shared_ptr are used.
 */
 #ifndef DAL_STATIC_STORED_OBJECTS_H__
 #define DAL_STATIC_STORED_OBJECTS_H__
@@ -76,11 +76,6 @@ Boost intrusive_ptr are used.
 
 
 #include "getfem/getfem_arch_config.h"
-#ifdef GETFEM_HAVE_BOOST
-#include <boost/intrusive_ptr.hpp>
-#else
-# include <getfem_boost/intrusive_ptr.hpp>
-#endif
 
 #ifdef GETFEM_HAVE_OPENMP
   #include <boost/atomic.hpp>
@@ -91,19 +86,45 @@ Boost intrusive_ptr are used.
   typedef bool  atomic_bool;
 #endif
 
-
+#define DAL_STORED_OBJECT_DEBUG 0
 
 namespace dal {
 
+
+#if DAL_STORED_OBJECT_DEBUG
+// Little tool for debug : detects the deleted static stored objects for
+// which the destructor is not called (i.e. a shared pointer is still stored
+// somewhere. Not thread safe.
+// Rule : Each potential stored object should called
+// DAL_STORED_OBJECT_DEBUG_CREATED(this, "name") in the constructor and
+// DAL_STORED_OBJECT_DEBUG_DESTROYED(this) in the destructor.
+  class static_stored_object;
+  void stored_debug_created(const static_stored_object *o,
+                            const std::string &name);
+  void stored_debug_added(const static_stored_object *o);
+  void stored_debug_deleted(const static_stored_object *o);
+  void stored_debug_destroyed(const static_stored_object *o,
+                              const std::string &name);
+# define DAL_STORED_OBJECT_DEBUG_CREATED(o, name) stored_debug_created(o, name)
+# define DAL_STORED_OBJECT_DEBUG_ADDED(o)   stored_debug_added(o)
+# define DAL_STORED_OBJECT_DEBUG_DELETED(o) stored_debug_deleted(o)
+# define DAL_STORED_OBJECT_DEBUG_DESTROYED(o, name) \
+                                            stored_debug_destroyed(o, name)
+#else
+# define DAL_STORED_OBJECT_DEBUG_CREATED(o, name)
+# define DAL_STORED_OBJECT_DEBUG_ADDED(o)
+# define DAL_STORED_OBJECT_DEBUG_DELETED(o)
+# define DAL_STORED_OBJECT_DEBUG_DESTROYED(o, name)
+#endif
+
   enum permanence { PERMANENT_STATIC_OBJECT = 0, // not deletable object
     STRONG_STATIC_OBJECT = 1,    // preferable not to delete it
     STANDARD_STATIC_OBJECT = 2,  // standard
     WEAK_STATIC_OBJECT = 3,      // delete it if necessary
-    AUTODELETE_STATIC_OBJECT = 4 // automatically deleted 
-    // when the last dependent object is deleted
+    AUTODELETE_STATIC_OBJECT = 4 // automatically deleted
+                                 // when the last dependent object is deleted
   };
 
-
   class static_stored_object_key {
   protected :
     virtual bool compare(const static_stored_object_key &) const {
@@ -124,121 +145,69 @@ namespace dal {
 
 
   template <typename var_type>
-  class simple_key : virtual public static_stored_object_key { 
-    var_type a;                                                     
-  public :                                                           
+  class simple_key : virtual public static_stored_object_key {
+    var_type a;
+  public :
     virtual bool compare(const static_stored_object_key &oo) const {
       const simple_key &o = dynamic_cast<const simple_key &>(oo);
-      if (a < o.a) return true; return false; 
+      if (a < o.a) return true; else return false;
     }
     simple_key(var_type aa) : a(aa) {}
   };
 
-#define DAL_SIMPLE_KEY(class_name, var_type)                         \
-  struct class_name : public dal::simple_key<var_type> {	     \
-  class_name(var_type aa) : dal::simple_key<var_type>(aa) {}	     \
+#define DAL_SIMPLE_KEY(class_name, var_type)                            \
+  struct class_name : public dal::simple_key<var_type> {                \
+  class_name(var_type aa) : dal::simple_key<var_type>(aa) {}            \
   }
 
-#define DAL_DOUBLE_KEY(class_name, var_type1, var_type2)	     \
-  struct class_name :						     \
-  public dal::simple_key<std::pair<var_type1,var_type2> > {	     \
-  class_name(var_type1 aa, var_type2 bb) :			     \
-  dal::simple_key<std::pair<var_type1,var_type2> >		     \
-  (std::make_pair(aa,bb)) {}					     \
+#define DAL_DOUBLE_KEY(class_name, var_type1, var_type2)                \
+  struct class_name :                                                   \
+  public dal::simple_key<std::pair<var_type1,var_type2> > {             \
+  class_name(var_type1 aa, var_type2 bb) :                              \
+  dal::simple_key<std::pair<var_type1,var_type2> >                      \
+  (std::make_pair(aa,bb)) {}                                            \
   }
 
-#define DAL_TRIPLE_KEY(class_name, var_type1, var_type2, var_type3)	\
-  struct class_name :							\
-  public dal::simple_key<std::pair<var_type1,				\
-  std::pair<var_type2,var_type3> > > { \
-  class_name(var_type1 aa, var_type2 bb, var_type3 cc) :		\
-  dal::simple_key<std::pair<var_type1,				\
-  std::pair<var_type2, var_type3> > >					\
-  (std::make_pair(aa,std::make_pair(bb,cc))) {}			\
+#define DAL_TRIPLE_KEY(class_name, var_type1, var_type2, var_type3)     \
+  struct class_name :                                                   \
+  public dal::simple_key<std::pair<var_type1,                           \
+                                   std::pair<var_type2,var_type3> > > { \
+  class_name(var_type1 aa, var_type2 bb, var_type3 cc) :                \
+    dal::simple_key<std::pair<var_type1,                                \
+                              std::pair<var_type2, var_type3> > >       \
+    (std::make_pair(aa,std::make_pair(bb,cc))) {}                       \
   }
 
 #define DAL_FOUR_KEY(class_name,var_type1,var_type2,var_type3,var_type4)\
-  struct class_name : public						\
-  dal::simple_key<std::pair						\
-  <var_type1, std::pair<var_type2, std::pair		\
-  <var_type3,var_type4> > > > {	\
-  class_name(var_type1 aa, var_type2 bb, var_type3 cc,var_type4 dd) : \
-  dal::simple_key<std::pair						\
-  <var_type1, std::pair<var_type2,			\
-  std::pair<var_type3,	\
-  var_type4> > > >	\
-  (std::make_pair(aa,std::make_pair(bb,std::make_pair(cc, dd)))) {} \
+  struct class_name : public                                            \
+  dal::simple_key<std::pair                                             \
+                  <var_type1, std::pair<var_type2, std::pair            \
+                                        <var_type3,var_type4> > > > {   \
+    class_name(var_type1 aa, var_type2 bb, var_type3 cc,var_type4 dd) : \
+      dal::simple_key<std::pair                                         \
+                      <var_type1, std::pair<var_type2,                  \
+                                            std::pair<var_type3,        \
+                                                      var_type4> > > >  \
+      (std::make_pair(aa,std::make_pair(bb,std::make_pair(cc, dd)))) {} \
   }
 
 
-
-  typedef const static_stored_object_key* pstatic_stored_object_key;
+  typedef std::shared_ptr<const static_stored_object_key>
+    pstatic_stored_object_key;
 
 
   /**
-  base class for reference-counted getfem objects (via
-  boost::intrusive_ptr).
-  The reference-counting is thread safe
-  @see dal_static_stored_objects.h
+  base class for static stored objects
   */
-  class static_stored_object 
-  {
-    mutable atomic_int pointer_ref_count_;
-  public :
-    static_stored_object(void) : pointer_ref_count_(0) {}
-    static_stored_object(const static_stored_object &) : pointer_ref_count_(0) {}
-    static_stored_object &operator =(const static_stored_object &) { return *this; }
-    virtual ~static_stored_object() { }
-    friend void intrusive_ptr_add_ref(const static_stored_object *o);
-    friend void intrusive_ptr_release(const static_stored_object *o);
-  };
-
-  typedef boost::intrusive_ptr<const static_stored_object>
-    pstatic_stored_object;
-
-  template<class T> boost::intrusive_ptr<const T>
-  stored_cast(pstatic_stored_object o) {
-    return boost::intrusive_ptr<const T>(dynamic_cast<const T *>(o.get()));
-  }
-
-
-#ifdef GETFEM_HAVE_OPENMP
-  inline void intrusive_ptr_add_ref(const static_stored_object * x)
-  {
-    x->pointer_ref_count_.fetch_add(1, boost::memory_order_relaxed);
-  }
-
-  inline void intrusive_ptr_release(const static_stored_object * x)
-  {
-    if (x->pointer_ref_count_.fetch_sub(1, boost::memory_order_release) == 1) 
-    {
-      boost::atomic_thread_fence(boost::memory_order_acquire);
-      delete x;
-    }
-  }
-#else
-  inline void intrusive_ptr_add_ref(const static_stored_object *o)
-  { 
-  	o->pointer_ref_count_++; 
-  }
-
-  inline void intrusive_ptr_release(const static_stored_object *o)
-  {    
-    if (--(o->pointer_ref_count_) == 0) delete o;  	
-  }
-#endif
+  class static_stored_object { public : virtual ~static_stored_object() {} };
 
+  typedef std::shared_ptr<const static_stored_object> pstatic_stored_object;
 
   pstatic_stored_object_key key_of_stored_object(pstatic_stored_object o);
 
   /** Gives a pointer to an object from a key pointer. */
   pstatic_stored_object search_stored_object(pstatic_stored_object_key k);
 
-  /** Gives a pointer to an object from a key reference. */
-  inline pstatic_stored_object
-    search_stored_object(const static_stored_object_key &k)
-  { return search_stored_object(&k); }
-
   /** Test if an object is stored*/
   bool exists_stored_object(pstatic_stored_object o);
 
@@ -250,7 +219,7 @@ namespace dal {
 
   /** Add an object with two optional dependencies. */
   void add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
-    permanence perm = STANDARD_STATIC_OBJECT);
+                         permanence perm = STANDARD_STATIC_OBJECT);
 
   inline void
     add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
@@ -261,8 +230,8 @@ namespace dal {
   }
 
   inline void
-    add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
-    pstatic_stored_object dep1, pstatic_stored_object dep2, 
+  add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
+    pstatic_stored_object dep1, pstatic_stored_object dep2,
     permanence perm = STANDARD_STATIC_OBJECT) {
       add_stored_object(k, o, perm);
       add_dependency(o, dep1);
@@ -270,10 +239,10 @@ namespace dal {
   }
 
   inline void
-    add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
-    pstatic_stored_object dep1, pstatic_stored_object dep2,
-    pstatic_stored_object dep3,
-    permanence perm = STANDARD_STATIC_OBJECT) {
+  add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
+                    pstatic_stored_object dep1, pstatic_stored_object dep2,
+                    pstatic_stored_object dep3,
+                    permanence perm = STANDARD_STATIC_OBJECT) {
       add_stored_object(k, o, perm);
       add_dependency(o, dep1);
       add_dependency(o, dep2);
@@ -281,10 +250,10 @@ namespace dal {
   }
 
   inline void
-    add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
-    pstatic_stored_object dep1, pstatic_stored_object dep2,
-    pstatic_stored_object dep3, pstatic_stored_object dep4,
-    permanence perm = STANDARD_STATIC_OBJECT) {
+  add_stored_object(pstatic_stored_object_key k, pstatic_stored_object o,
+                    pstatic_stored_object dep1, pstatic_stored_object dep2,
+                    pstatic_stored_object dep3, pstatic_stored_object dep4,
+                    permanence perm = STANDARD_STATIC_OBJECT) {
       add_stored_object(k, o, perm);
       add_dependency(o, dep1);
       add_dependency(o, dep2);
@@ -293,7 +262,8 @@ namespace dal {
   }
 
   /** Delete an object and the object which depend on it. */
-  void del_stored_object(pstatic_stored_object o, bool ignore_unstored=false);
+  void del_stored_object(const pstatic_stored_object &o,
+                         bool ignore_unstored=false);
 
   /** Delete all the object whose permanence is greater or equal to perm. */
   void del_stored_objects(int perm);
@@ -307,7 +277,7 @@ namespace dal {
   /** Delete a list of objects and their dependencies*/
   void del_stored_objects(std::list<pstatic_stored_object> &to_delete,
     bool ignore_unstored);
-  
+
   /** Test the validity of the whole global storage */
   void test_stored_objects(void);
 
@@ -321,9 +291,9 @@ namespace dal {
     std::set<pstatic_stored_object> dependencies;
     enr_static_stored_object(pstatic_stored_object o, permanence perma)
       : p(o), perm(perma) {valid = true;}
-    enr_static_stored_object(void)
-      : p(0), perm(STANDARD_STATIC_OBJECT) {valid = true;}
-    enr_static_stored_object(const enr_static_stored_object& enr_o) 
+    enr_static_stored_object()
+      : perm(STANDARD_STATIC_OBJECT) {valid = true;}
+    enr_static_stored_object(const enr_static_stored_object& enr_o)
       : p(enr_o.p), perm(enr_o.perm), dependent_object(enr_o.dependent_object),
       dependencies(enr_o.dependencies){valid = static_cast<bool>(enr_o.perm);}
   };
@@ -341,21 +311,16 @@ namespace dal {
 
 
   /** Table of stored objects. Thread safe, uses thread specific mutexes. */
-  struct stored_object_tab : 
-    public std::map<enr_static_stored_object_key, enr_static_stored_object>
-  {
-  
-    struct stored_key_tab : 
-      public std::map<pstatic_stored_object,pstatic_stored_object_key> 
-    {
-      ~stored_key_tab() 
-      {
-        for (iterator it = begin(); it != end(); ++it) delete it->second;
-      }
-    };
+  struct stored_object_tab :
+    public std::map<enr_static_stored_object_key, enr_static_stored_object> {
+
+    typedef std::map<pstatic_stored_object,pstatic_stored_object_key>
+      stored_key_tab;
 
     stored_object_tab();
-    pstatic_stored_object search_stored_object(pstatic_stored_object_key k) const;
+    ~stored_object_tab();
+    pstatic_stored_object
+      search_stored_object(pstatic_stored_object_key k) const;
     bool has_dependent_objects(pstatic_stored_object o) const;
     bool exists_stored_object(pstatic_stored_object o) const;
     //adding the object to the storage on the current thread
@@ -364,22 +329,22 @@ namespace dal {
 
     iterator iterator_of_object_(pstatic_stored_object o);
     //delete o2 from the dependency list of o1
-    //true if successfull, false if o1 is not 
+    //true if successfull, false if o1 is not
     //on this thread
     bool del_dependency_(pstatic_stored_object o1,
     pstatic_stored_object o2);
     //delete o1 from the dependent list of o2
-    //true if successfull, false if o1 is not 
+    //true if successfull, false if o1 is not
     //on this thread
     bool del_dependent_(pstatic_stored_object o1,
     pstatic_stored_object o2);
     //add o2 to the dependency list of o1
-    //true if successfull, false if o1 is not 
+    //true if successfull, false if o1 is not
     //on this thread
     bool add_dependency_(pstatic_stored_object o1,
     pstatic_stored_object o2);
     //add o1 to the dependent list of o2
-    //true if successfull, false if o1 is not 
+    //true if successfull, false if o1 is not
     //on this thread
     bool add_dependent_(pstatic_stored_object o1,
     pstatic_stored_object o2);
@@ -406,11 +371,12 @@ namespace dal {
       iterator ite = stored_objects.end();
 
       for(iterator it = itb; it != ite; ++it){
-        const OBJECT_TYPE *p_object =  dal::stored_cast<OBJECT_TYPE>(it->second.p).get();
+        const OBJECT_TYPE *p_object
+          = std::dynamic_pointer_cast<const OBJECT_TYPE>(it->second.p).get();
         if(p_object != 0) delete_object_list.push_back(it->second.p);
-      }    
+      }
     }
-    else{    
+    else{
       for(size_t thread = 0; thread<getfem::num_threads();thread++)
       {
         stored_object_tab& stored_objects
@@ -420,7 +386,8 @@ namespace dal {
         iterator ite = stored_objects.end();
 
         for(iterator it = itb; it != ite; ++it){
-          const OBJECT_TYPE *p_object =  dal::stored_cast<OBJECT_TYPE>(it->second.p).get();
+          const OBJECT_TYPE *p_object
+            = std::dynamic_pointer_cast<const OBJECT_TYPE>(it->second.p).get();
           if(p_object != 0) delete_object_list.push_back(it->second.p);
         }
       }
