@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 2000-2016 Yves Renard
+ Copyright (C) 2000-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -76,14 +76,19 @@
                close to a polar integration with respect to vertex IP1.
                if IM1 is an integration method on a tetrahedron, gives an
                integration method on a tetrahedron which is close to a
-               cylindrical integration with respect to vertex IP1 (does not work very well).
+               cylindrical integration with respect to vertex IP1
+               (does not work very well).
                if IM1 is an integration method on a prism. Gives an integration
                method on a tetrahedron which is close to a
                cylindrical integration with respect to vertex IP1.
+
    - "IM_QUASI_POLAR(IM1, IP1, IP2)"   : IM1 should be an integration method
                on a prism. Gives an integration method on a tetrahedron which
                is close to a cylindrical integration with respect to IP1-IP2
                axis.
+
+   - "IM_PYRAMID_COMPOSITE(IM1)"       : Composite integration for a pyramid
+                                         decomposed into two tetrahedrons
 */
 #ifndef GETFEM_INTEGRATION_H__
 #define GETFEM_INTEGRATION_H__
