@@ -1,7 +1,7 @@
 /* -*- c++ -*- (enables emacs c++ mode) */
 /*===========================================================================
 
- Copyright (C) 1999-2016 Yves Renard
+ Copyright (C) 1999-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -88,7 +88,7 @@ namespace getfem {
     bool operator < (const iterator &i) const
     { return (it < i.it) && (ii < i.ii); }
 
-    tab_scal_to_vect_iterator(void) {}
+    tab_scal_to_vect_iterator() {}
     tab_scal_to_vect_iterator(const ITER &iter, dim_type n, dim_type i)
       : it(iter), N(n), ii(i) { }
 
@@ -120,20 +120,20 @@ namespace getfem {
 
   public :
 
-    bool empty(void) const { return it == ite; }
-    size_type size(void) const { return (ite - it) * N; }
+    bool empty() const { return it == ite; }
+    size_type size() const { return (ite - it) * N; }
 
-    const_iterator begin(void) const { return iterator(it, N, 0); }
-    const_iterator end(void) const { return iterator(ite, N, 0); }
-    const_reverse_iterator rbegin(void) const
+    const_iterator begin() const { return iterator(it, N, 0); }
+    const_iterator end() const { return iterator(ite, N, 0); }
+    const_reverse_iterator rbegin() const
     { return const_reverse_iterator(end()); }
-    const_reverse_iterator rend(void) const
+    const_reverse_iterator rend() const
     { return const_reverse_iterator(begin()); }
 
-    value_type front(void) const { return *begin(); }
-    value_type back(void) const { return *(--(end())); }
+    value_type front() const { return *begin(); }
+    value_type back() const { return *(--(end())); }
 
-    tab_scal_to_vect(void) : N(0) {}
+    tab_scal_to_vect() : N(0) {}
     tab_scal_to_vect(const CONT &cc, dim_type n)
       : it(cc.begin()), ite(cc.end()), N(n) {}
 
@@ -153,13 +153,14 @@ namespace getfem {
     void copy_from(const mesh_fem &mf); /* Remember to change copy_from if
                                            adding components to mesh_fem */
 
-    dal::dynamic_array<pfem> f_elems;
+    std::vector<pfem> f_elems;
     dal::bit_vector fe_convex;
     const mesh *linked_mesh_;
     REDUCTION_MATRIX R_;
     EXTENSION_MATRIX E_;
     mutable bgeot::mesh_structure dof_structure;
     mutable bool dof_enumeration_made;
+    mutable bool is_uniform_, is_uniformly_vectorized_;
     mutable size_type nb_total_dof;
     pfem auto_add_elt_pf; /* fem for automatic addition                   */
                        /* of element option. (0 = no automatic addition)  */
@@ -180,24 +181,24 @@ namespace getfem {
     typedef tab_scal_to_vect<mesh::ind_cv_ct> ind_dof_ct;
     typedef tab_scal_to_vect<mesh::ind_pt_face_ct> ind_dof_face_ct;
 
-    void update_from_context(void) const;
+    void update_from_context() const;
 
-    gmm::uint64_type version_number(void) const
+    gmm::uint64_type version_number() const
     { context_check(); return v_num; }
 
     /** Get the set of convexes where a finite element has been assigned.
      */
-    inline const dal::bit_vector &convex_index(void) const
+    inline const dal::bit_vector &convex_index() const
     { context_check(); return fe_convex; }
 
     /// Return true if a reduction matrix is applied to the dofs.
-    bool is_reduced(void) const { return use_reduction; }
+    bool is_reduced() const { return use_reduction; }
 
     /// Return the reduction matrix applied to the dofs.
-    const REDUCTION_MATRIX &reduction_matrix(void) const { return R_; }
+    const REDUCTION_MATRIX &reduction_matrix() const { return R_; }
 
     /// Return the extension matrix corresponding to reduction applied (RE=I).
-    const EXTENSION_MATRIX &extension_matrix(void) const { return E_; }
+    const EXTENSION_MATRIX &extension_matrix() const { return E_; }
 
     /** Allows to set the reduction and the extension matrices.
      * Should satify (RR*EE=I). */
@@ -274,7 +275,10 @@ namespace getfem {
 
 
     /// Return a reference to the underlying mesh.
-    const mesh &linked_mesh(void) const { return *linked_mesh_; }
+    const mesh &linked_mesh() const { return *linked_mesh_; }
+
+    virtual bool is_uniform() const;
+    virtual bool is_uniformly_vectorized() const;
 
     /** Set the degree of the fem for automatic addition
      *  of element option. K=-1 disables the automatic addition.
@@ -420,7 +424,7 @@ namespace getfem {
      *  vectorization due to qdim nor the optional reduction.
      */
     virtual pfem fem_of_element(size_type cv) const
-    { return  f_elems[cv]; }
+    { return ((cv < f_elems.size()) ? f_elems[cv] : 0); }
     /** Give an array of the dof numbers a of convex.
      *  @param cv the convex number.
      *  @return a pseudo-container of the dof number.
@@ -432,6 +436,9 @@ namespace getfem {
     }
     ind_dof_ct ind_dof_of_element(size_type cv) const IS_DEPRECATED
     { return ind_basic_dof_of_element(cv); }
+    virtual const bgeot::mesh_structure::ind_cv_ct &
+    ind_scalar_basic_dof_of_element(size_type cv) const
+    { return dof_structure.ind_points_of_convex(cv); }
     /** Give an array of the dof numbers lying of a convex face (all
               degrees of freedom whose associated base function is non-zero
         on the convex face).
@@ -454,7 +461,7 @@ namespace getfem {
         @param f the face number.
     */
     virtual size_type nb_basic_dof_of_face_of_element(size_type cv,
-                                              short_type f) const {
+						      short_type f) const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
       pfem pf = f_elems[cv];
       return dof_structure.structure_of_convex(cv)->nb_points_of_face(f)
@@ -520,20 +527,20 @@ namespace getfem {
     virtual void get_global_dof_index(std::vector<size_type> &ind) const;
     /** Renumber the degrees of freedom. You should not have
      * to call this function, as it is done automatically */
-    virtual void enumerate_dof(void) const;
+    virtual void enumerate_dof() const;
 
 #if GETFEM_PARA_LEVEL > 1
-    void enumerate_dof_para(void)const;
+    void enumerate_dof_para()const;
 #endif
 
     /** Return the total number of basic degrees of freedom (before the
      * optional reduction). */
-    virtual size_type nb_basic_dof(void) const {
+    virtual size_type nb_basic_dof() const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
       return nb_total_dof;
     }
     /// Return the total number of degrees of freedom.
-    virtual size_type nb_dof(void) const {
+    virtual size_type nb_dof() const {
       context_check(); if (!dof_enumeration_made) enumerate_dof();
       return use_reduction ? gmm::mat_nrows(R_) : nb_total_dof;
     }
@@ -555,8 +562,8 @@ namespace getfem {
 
     void set_dof_partition(size_type cv, unsigned partition_num) {
       if (dof_partition.empty() && partition_num == 0) return;
-      if (dof_partition.size() < linked_mesh().convex_index().last_true()+1)
-        dof_partition.resize(linked_mesh().convex_index().last_true()+1);
+      if (dof_partition.size() < linked_mesh().nb_allocated_convex())
+        dof_partition.resize(linked_mesh().nb_allocated_convex());
       if (dof_partition.at(cv) != partition_num) {
         dof_partition[cv] = partition_num;
         dof_enumeration_made = false;
@@ -570,7 +577,7 @@ namespace getfem {
     size_type memsize() const {
       return dof_structure.memsize() +
         sizeof(mesh_fem) - sizeof(bgeot::mesh_structure) +
-        f_elems.memsize() + fe_convex.memsize();
+        f_elems.size() * sizeof(pfem) + fe_convex.memsize();
     }
     void init_with_mesh(const mesh &me, dim_type Q = 1);
     /** Build a new mesh_fem. A mesh object must be supplied.
@@ -578,12 +585,12 @@ namespace getfem {
         @param Q the Q dimension (see mesh_fem::get_qdim).
     */
     explicit mesh_fem(const mesh &me, dim_type Q = 1);
-    mesh_fem(void);
+    mesh_fem();
     mesh_fem(const mesh_fem &mf);
     mesh_fem &operator=(const mesh_fem &mf);
 
     virtual ~mesh_fem();
-    virtual void clear(void);
+    virtual void clear();
     /** Read the mesh_fem from a stream.
         @param ist the stream.
      */
@@ -625,26 +632,36 @@ namespace getfem {
   /** Given a mesh_fem @param mf and a vector @param vec of size equal to
    *  mf.nb_basic_dof(), the output vector @param coeff will contain the
    *  values of @param vec corresponding to the basic dofs of element
-   *  @param cv . The size of @param coeff is adjusted if necessary.
+   *  @param cv. The size of @param coeff is adjusted if necessary.
    */
   template <typename VEC1, typename VEC2>
   void slice_vector_on_basic_dof_of_element(const mesh_fem &mf,
                                             const VEC1 &vec,
-                                            size_type cv, VEC2 &coeff) {
-    size_type nbdof = mf.nb_basic_dof();
-    size_type qmult = gmm::vect_size(vec) / nbdof;
-    GMM_ASSERT1(gmm::vect_size(vec) == qmult * nbdof, "Bad dof vector size");
-    size_type cvnbdof = mf.nb_basic_dof_of_element(cv);
-    gmm::resize(coeff, cvnbdof*qmult);
-    mesh_fem::ind_dof_ct::const_iterator
-      itdof = mf.ind_basic_dof_of_element(cv).begin();
-    if (qmult == 1) {
-      for (size_type k = 0; k < cvnbdof; ++k, ++itdof)
-          coeff[k] = vec[*itdof];
+                                            size_type cv, VEC2 &coeff,
+					    size_type qmult1 = size_type(-1),
+					    size_type qmult2 = size_type(-1)) {
+    if (qmult1 == size_type(-1)) {
+      size_type nbdof = mf.nb_basic_dof();
+      qmult1 = gmm::vect_size(vec) / nbdof;
+      GMM_ASSERT1(gmm::vect_size(vec) == qmult1 * nbdof, "Bad dof vector size");
+    }
+    if (qmult2 == size_type(-1)) {
+      qmult2 = mf.get_qdim();
+      if (qmult2 > 1) qmult2 /= mf.fem_of_element(cv)->target_dim();
+    }
+    size_type qmultot = qmult1*qmult2;
+    auto &ct = mf.ind_scalar_basic_dof_of_element(cv);
+    gmm::resize(coeff, ct.size()*qmultot);
+    
+    auto it = ct.begin();
+    auto itc = coeff.begin();
+    if (qmultot == 1) {
+      for (; it != ct.end(); ++it) *itc++ = vec[*it];
     } else {
-      for (size_type k = 0; k < cvnbdof; ++k, ++itdof)
-        for (size_type l = 0; l < qmult; ++l)
-          coeff[l+k*qmult] = vec[l+(*itdof)*qmult];
+      for (; it != ct.end(); ++it) {
+	auto itv = vec.begin()+(*it)*qmult1;
+	for (size_type m = 0; m < qmultot; ++m) *itc++ = *itv++;
+      }
     }
   }
 
