@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2003-2016 Julien Pommier
+ Copyright (C) 2003-2017 Julien Pommier
 
  This file is a part of GetFEM++
 
@@ -162,16 +162,16 @@ namespace getfem {
               "of size " << rn << " with '"
               << s << "'");
           }
-          //cerr << "ts = " << child(n).ranges() << ", red="
-          //     << red[n].second << "\n";
-          bgeot::tensor_reduction::diag_shape(ts, red[n].second);
-          // cerr << "REDUCTION '" << red[n].second
-          //        << "' -> sending required to child#" << int(n)
-          //	<< " " << child(n).name() << ":" << endl;
-          // cerr << ts << endl;
-          child(n).merge_required_shape(ts);
-          // cerr << "------>required shape is now: "
-          //      << child(n).required_shape() << endl;
+	//cerr << "ts = " << child(n).ranges() << ", red="
+	//     << red[n].second << "\n";
+	bgeot::tensor_reduction::diag_shape(ts, red[n].second);
+	// cerr << "REDUCTION '" << red[n].second
+	//        << "' -> sending required to child#" << int(n)
+	//	<< " " << child(n).name() << ":" << endl;
+	// cerr << ts << endl;
+	child(n).merge_required_shape(ts);
+	// cerr << "------>required shape is now: "
+	//      << child(n).required_shape() << endl;
       }
     }
 
@@ -280,9 +280,8 @@ namespace getfem {
       if ((shape_updated_ = child(0).is_shape_updated())) {
         if (reorder.size() != child(0).ranges().size())
           ASM_THROW_TENSOR_ERROR("can't reorder tensor '" << name()
-          << "' of dimensions " 
-          << child(0).ranges() << 
-          " with this permutation: " << reorder);
+				 << "' of dimensions " << child(0).ranges() << 
+				 " with this permutation: " << vref(reorder));
         r_.resize(reorder.size());
         std::fill(r_.begin(), r_.end(), dim_type(-1));
 
@@ -1462,9 +1461,9 @@ namespace getfem {
       }
       if (check_permut.first_false() != reorder.size()) {
         cerr << check_permut << endl;
-        cerr << reorder << endl;
+        cerr << vref(reorder) << endl;
         ASM_THROW_PARSE_ERROR("you did not give a real permutation:"
-          << reorder);
+			      << vref(reorder));
       }
       t = tnode(record(std::make_unique<ATN_permuted_tensor>(*t.tensor(), reorder)));
     }
