@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 1999-2016 Yves Renard
+ Copyright (C) 1999-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -29,7 +29,7 @@ namespace getfem {
     cvr = bfem->ref_convex(0);
     dim_ = cvr->structure()->dim();
     is_equiv = true; real_element_defined = true;
-    is_polycomp = is_pol = is_lag = false;
+    is_polycomp = is_pol = is_lag = is_standard_fem = false;
     es_degree = 5; /* humm ... */
     ntarget_dim = bfem->target_dim();
 
@@ -201,10 +201,7 @@ namespace getfem {
   
   void fem_level_set::real_hess_base_value(const fem_interpolation_context &c,
                                            base_tensor &t, bool) const {
-    // bgeot::multi_index mi(4);
-    // mi[3] = mi[2] = short_type(c.N()); mi[1] = target_dim();
-    // mi[0] = short_type(nb_base(0));
-    t.adjust_sizes(nb_base(0), target_dim(), c.N(), c.N());
+    t.adjust_sizes(nb_base(0), target_dim(), gmm::sqr(c.N()));
     fem_interpolation_context c0 = c;
     if (c0.have_pfp())
       c0.set_pfp(fem_precomp(bfem, c0.pfp()->get_ppoint_tab(), c0.pfp()));
@@ -217,17 +214,15 @@ namespace getfem {
     std::vector<bool> zid;
     find_zone_id(c, zid, c.xfem_side());
 
-    for (dim_type i = 0; i < c.N() ; ++i) {
-      for (dim_type j = 0; j < c.N() ; ++j) {
-	for (dim_type q = 0; q < target_dim(); ++q) {
-	  unsigned cnt = 0;
-	  for (size_type d = 0; d < bfem->nb_dof(0); ++d, ++itf) {
-	    if (dofzones[d]) { /* enriched dof ? */
-	      for (size_type k = 0; k < dofzones[d]->size(); ++k, ++cnt)
-		*it++ = zid[cnt] ? *itf : 0;
-	    } else *it++ = *itf;
-	  }
-	}
+    dim_type NNdim = dim_type(gmm::sqr(c.N())*target_dim());
+    for (dim_type ijq = 0; ijq < NNdim ; ++ijq) {
+      unsigned cnt = 0;
+      for (size_type d = 0; d < bfem->nb_dof(0); ++d, ++itf) {
+        if (dofzones[d]) /* enriched dof ? */
+          for (size_type k = 0; k < dofzones[d]->size(); ++k, ++cnt)
+            *it++ = zid[cnt] ? *itf : 0;
+        else
+          *it++ = *itf;
       }
     }
     assert(it == t.end());
