@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2004-2016 Yves Renard
+ Copyright (C) 2004-2017 Yves Renard
  Copyright (C) 2016      Konstantinos Poulios
 
  This file is a part of GetFEM++
@@ -55,13 +55,17 @@ namespace getfem {
   // Implementation of global_function_parser
 
   scalar_type global_function_parser::val(const base_node &pt) const {
-    gmm::copy(pt, pt_);
-    const bgeot::base_tensor &t = f_val.eval();
+    const bgeot::base_tensor &t = tensor_val(pt);
     GMM_ASSERT1(t.size() == 1, "Wrong size of expression result "
                 << f_val.expression());
     return t[0];
   }
 
+  const base_tensor &global_function_parser::tensor_val(const base_node &pt) const {
+    gmm::copy(pt, pt_);
+    return f_val.eval();
+  }
+
   void global_function_parser::grad(const base_node &pt, base_small_vector &g) const {
     g.resize(dim_);
     gmm::copy(pt, pt_);
@@ -134,18 +138,60 @@ namespace getfem {
     }
   }
 
+  bool global_function_sum::is_in_support(const base_node &p) const {
+    for (const auto &f : functions)
+      if (f->is_in_support(p)) return true;
+    return false;
+  }
+
+  void global_function_sum::bounding_box
+  (base_node &bmin_, base_node &bmax_) const {
+    if (functions.size() > 0)
+      functions[0]->bounding_box(bmin_, bmax_);
+    base_node bmin0(dim()), bmax0(dim());
+    for (const auto &f : functions) {
+      f->bounding_box(bmin0, bmax0);
+      for (size_type i=0; i < dim(); ++i) {
+        if (bmin0[i] < bmin_[i]) bmin_[i] = bmin0[i];
+        if (bmax0[i] > bmax_[i]) bmax_[i] = bmax0[i];
+      }
+    }
+  }
+
   global_function_sum::global_function_sum(const std::vector<pglobal_function> &funcs)
     : global_function((funcs.size() > 0) ? funcs[0]->dim() : 0), functions(funcs) {
     for (const auto &f : functions)
       GMM_ASSERT1(f->dim() == dim(), "Incompatible dimensions among the provided"
                                      " global functions");
   }
+
   global_function_sum::global_function_sum(pglobal_function f1, pglobal_function f2)
     : global_function(f1->dim()), functions(2) {
     functions[0] = f1;
     functions[1] = f2;
-    GMM_ASSERT1(f2->dim() == dim(), "Incompatible dimensions between the provided"
-                                    " global functions");
+    GMM_ASSERT1(f1->dim() == dim() && f2->dim() == dim(),
+                "Incompatible dimensions between the provided global functions");
+  }
+
+  global_function_sum::global_function_sum(pglobal_function f1, pglobal_function f2,
+                                           pglobal_function f3)
+    : global_function(f1->dim()), functions(3) {
+    functions[0] = f1;
+    functions[1] = f2;
+    functions[2] = f3;
+    GMM_ASSERT1(f1->dim() == dim() && f2->dim() == dim() && f3->dim() == dim(),
+                "Incompatible dimensions between the provided global functions");
+  }
+
+  global_function_sum::global_function_sum(pglobal_function f1, pglobal_function f2,
+                                           pglobal_function f3, pglobal_function f4)
+    : global_function(f1->dim()), functions(4) {
+    functions[0] = f1;
+    functions[1] = f2;
+    functions[2] = f3;
+    functions[3] = f4;
+    GMM_ASSERT1(f1->dim() == dim() && f2->dim() == dim() && f3->dim() == dim(),
+                "Incompatible dimensions between the provided global functions");
   }
 
 
@@ -182,6 +228,23 @@ namespace getfem {
     gmm::rank_one_update(h, g2, g1);
   }
 
+  bool global_function_product::is_in_support(const base_node &p) const {
+    return f1->is_in_support(p) && f2->is_in_support(p);
+  }
+
+  void global_function_product::bounding_box
+  (base_node &bmin_, base_node &bmax_) const {
+    base_node bmin0(dim()), bmax0(dim());
+    f1->bounding_box(bmin0, bmax0);
+    f2->bounding_box(bmin_, bmax_);
+    for (size_type i=0; i < dim(); ++i) {
+      if (bmin0[i] > bmin_[i]) bmin_[i] = bmin0[i];
+      if (bmax0[i] < bmax_[i]) bmax_[i] = bmax0[i];
+      if (bmin_[i] > bmax_[i])
+        GMM_WARNING1("Global function product with vanishing basis function");
+    }
+  }
+
   global_function_product::global_function_product(pglobal_function f1_, pglobal_function f2_)
     : global_function(f1_->dim()), f1(f1_), f2(f2_) {
     GMM_ASSERT1(f2->dim() == dim(), "Incompatible dimensions between the provided"
