@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 1999-2016 Yves Renard
+ Copyright (C) 1999-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -90,7 +90,7 @@ namespace getfem {
                 giv.invert(pt, barycentre);
 
         
-                if (gmm::abs(pgt->convex_ref()->is_in_face(f, barycentre)) < 0.1)
+                if (gmm::abs(pgt->convex_ref()->is_in_face(f,barycentre)) < 0.001)
                   r.add(icv[jc], fsub);
               }
             }
@@ -115,7 +115,7 @@ namespace getfem {
               giv.invert(pt, barycentre);
         
               for (short_type f = 0; f < pgt->structure()->nb_faces(); ++f)
-                if (gmm::abs(pgt->convex_ref()->is_in_face(f, barycentre)) < 0.1)
+                if (gmm::abs(pgt->convex_ref()->is_in_face(f,barycentre)) < 0.001)
                   { r.add(ic, f); break; }
           }
         }
@@ -156,7 +156,7 @@ namespace getfem {
       else {
         int ne = int(nb_convex());
         std::vector<int> xadj(ne+1), adjncy, numelt(ne), npart(ne);
-        std::vector<int> indelt(convex_index().last_true()+1);
+        std::vector<int> indelt(nb_allocated_convex());
         
         double t_ref = MPI_Wtime();
 
@@ -217,10 +217,9 @@ namespace getfem {
   }
 #endif
 
-  void mesh::optimize_structure() {
+  void mesh::optimize_structure(bool with_renumbering) {
     pts.resort();
-    size_type i, j;
-    j = nb_convex();
+    size_type i, j = nb_convex(), nbc = j;
     for (i = 0; i < j; i++)
       if (!convex_tab.index_valid(i))
         swap_convex(i, convex_tab.ind_last());
@@ -231,14 +230,20 @@ namespace getfem {
         while (i < j && j != ST_NIL && !(pts.index()[j])) --j;
         if (i < j && j != ST_NIL ) swap_points(i, j);
       }
-  }
-
-  const std::vector<size_type> &mesh::cuthill_mckee_ordering() const {
-    if (!cuthill_mckee_uptodate) {
-      bgeot::cuthill_mckee_on_convexes(*this, cmk_order);
-      cuthill_mckee_uptodate = true;
+    if (with_renumbering) { // Could be optimized no using only swap_convex
+      std::vector<size_type> cmk, iord(nbc), iordinv(nbc);
+      for (i = 0; i < nbc; ++i) iord[i] = iordinv[i] = i;
+      
+      bgeot::cuthill_mckee_on_convexes(*this, cmk);
+      for (i = 0; i < nbc; ++i) {
+	j = iordinv[cmk[i]];
+	if (i != j) {
+	  swap_convex(i, j);
+	  std::swap(iord[i], iord[j]);
+	  std::swap(iordinv[iord[i]], iordinv[iord[j]]);
+	}
+      }
     }
-    return cmk_order;
   }
 
   void mesh::translation(const base_small_vector &V)
@@ -293,6 +298,12 @@ namespace getfem {
     return add_simplex(3, &(ipt[0]));
   }
 
+  size_type mesh::add_pyramid(size_type a, size_type b,
+                                  size_type c, size_type d, size_type e) {
+    size_type ipt[5]; ipt[0] = a; ipt[1] = b; ipt[2] = c; ipt[3] = d; ipt[4] = e;
+    return add_convex(bgeot::pyramidal_geotrans(1), &(ipt[0]));
+  }
+
   size_type mesh::add_tetrahedron_by_points
   (const base_node &p1, const base_node &p2,
    const base_node &p3, const base_node &p4) {
@@ -870,6 +881,28 @@ namespace getfem {
     return mrr;
   }
 
+  mesh_region select_convexes_in_box(const mesh &m, const mesh_region &mr,
+                                     const base_node &pt1, const base_node &pt2) {
+    mesh_region mrr;
+    size_type N = m.dim();
+    GMM_ASSERT1(pt1.size() == N && pt2.size() == N, "Wrong dimensions"); 
+    for (getfem::mr_visitor i(mr, m); !i.finished(); ++i)
+      if (i.f() == short_type(-1)) {
+        bgeot::mesh_structure::ind_cv_ct pt = m.ind_points_of_convex(i.cv());
+
+        bool is_in = true;
+        for (bgeot::mesh_structure::ind_cv_ct::iterator it = pt.begin();
+             it != pt.end(); ++it) {
+          for (size_type j = 0; j < N; ++j)
+            if (m.points()[*it][j] < pt1[j] || m.points()[*it][j] > pt2[j])
+              { is_in = false; break; }
+          if (!is_in) break;
+        }
+        if (is_in) mrr.add(i.cv());
+      }
+    return mrr;
+  }
+
   void extrude(const mesh& in, mesh& out, size_type nb_layers, short_type degree) {
     dim_type dim = in.dim();
     base_node pt(dim+1);
@@ -917,9 +950,12 @@ namespace bgeot {
 namespace getfem {
 
   bool mesh::edge::operator <(const edge &e) const {
-    if (i0 < e.i0) return true; if (i0 > e.i0) return false;
-    if (i1 < e.i1) return true; if (i1 > e.i1) return false;
-    if (i2 < e.i2) return true; return false;
+    if (i0 < e.i0) return true;
+    if (i0 > e.i0) return false;
+    if (i1 < e.i1) return true;
+    if (i1 > e.i1) return false;
+    if (i2 < e.i2) return true;
+    return false;
   }
 
   void mesh::Bank_sup_convex_from_green(size_type i) {
