@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 1999-2016 Yves Renard
+ Copyright (C) 1999-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -27,7 +27,7 @@
 
 namespace getfem {
 
-  void mesh_fem::update_from_context(void) const {
+  void mesh_fem::update_from_context() const {
     for (dal::bv_visitor i(fe_convex); !i.finished(); ++i) {
       if (linked_mesh_->convex_index().is_in(i)) {
         if (v_num_update < linked_mesh_->convex_version_number(i)) {
@@ -142,11 +142,21 @@ namespace getfem {
                   "Incompatibility between Qdim=" << int(Qdim) <<
                   " and target_dim " << int(pf->target_dim()) << " of " <<
                   name_of_fem(pf));
-      if (!fe_convex.is_in(cv) || f_elems[cv] != pf) {
-        fe_convex.add(cv);
-        f_elems[cv] = pf;
-        dof_enumeration_made = false;
+
+      
+      if (cv == f_elems.size()) {
+	f_elems.push_back(pf);
+	fe_convex.add(cv);
+	dof_enumeration_made = false;
         touch(); v_num = act_counter();
+      } else {
+	if (cv > f_elems.size()) f_elems.resize(cv+1);
+	if (!fe_convex.is_in(cv) || f_elems[cv] != pf) {
+	  fe_convex.add(cv);
+	  f_elems[cv] = pf;
+	  dof_enumeration_made = false;
+	  touch(); v_num = act_counter();
+	}
       }
     }
   }
@@ -290,22 +300,34 @@ namespace getfem {
     }
   }
 
+  bool mesh_fem::is_uniform() const {
+    context_check(); if (!dof_enumeration_made) enumerate_dof();
+    return is_uniform_;
+  }
+
+  bool mesh_fem::is_uniformly_vectorized() const {
+    context_check(); if (!dof_enumeration_made) enumerate_dof();
+    return is_uniformly_vectorized_;
+  }
+
   /// Enumeration of dofs
-  void mesh_fem::enumerate_dof(void) const {
+  void mesh_fem::enumerate_dof() const {
     bgeot::index_node_pair ipt;
+    is_uniform_ = true;
+    is_uniformly_vectorized_ = (get_qdim() > 1);
     GMM_ASSERT1(linked_mesh_ != 0, "Uninitialized mesh_fem");
     context_check();
     if (fe_convex.card() == 0)
       { dof_enumeration_made = true; nb_total_dof = 0; return; }
-
-    // Gives the Cuthill McKee ordering to iterate on elements
-    const std::vector<size_type> &cmk = linked_mesh().cuthill_mckee_ordering();
+    pfem first_pf = f_elems[fe_convex.first_true()];
+    if (first_pf && first_pf->is_on_real_element()) is_uniform_ = false;
+    if (first_pf && first_pf->target_dim() > 1) is_uniformly_vectorized_=false;
 
     // Dof counter
     size_type nbdof = 0;
 
     // Information stored per element
-    size_type nb_max_cv = linked_mesh().convex_index().last_true()+1;
+    size_type nb_max_cv = linked_mesh().nb_allocated_convex();
     std::vector<bgeot::kdtree> dof_nodes(nb_max_cv);
     std::vector<scalar_type> elt_car_sizes(nb_max_cv);
     std::vector<std::map<fem_dof, size_type, dof_comp_>> dof_sorts(nb_max_cv);
@@ -326,7 +348,8 @@ namespace getfem {
     bgeot::pgeometric_trans pgt_old = 0;
     bgeot::pgeotrans_precomp pgp = 0;
 
-    for (size_type cv : cmk) {
+    for (dal::bv_visitor cv(linked_mesh().convex_index());
+      	 !cv.finished(); ++cv) {
       if (fe_convex.is_in(cv)) {
 	gmm::copy(linked_mesh().points_of_convex(cv)[0], bmin);
 	gmm::copy(bmin, bmax);
@@ -342,10 +365,13 @@ namespace getfem {
     }
 
     dal::bit_vector cv_done;
-
-    for (size_type cv : cmk) { // Loop on elements
+    
+    for (dal::bv_visitor cv(linked_mesh().convex_index());
+	 !cv.finished(); ++cv) { // Loop on elements
       if (!fe_convex.is_in(cv)) continue;
       pfem pf = fem_of_element(cv);
+      if (pf != first_pf) is_uniform_ = false;
+      if (pf->target_dim() > 1) is_uniformly_vectorized_ = false;
       bgeot::pgeometric_trans pgt = linked_mesh().trans_of_convex(cv);
       bgeot::pstored_point_tab pspt = pf->node_tab(cv);
       if (pgt != pgt_old || pspt != pspt_old)
@@ -433,9 +459,10 @@ namespace getfem {
     set_reduction_matrices(RR, gmm::transposed(RR));
   }
 
-  void mesh_fem::clear(void) {
+  void mesh_fem::clear() {
     fe_convex.clear();
     dof_enumeration_made = false;
+    is_uniform_ = true;
     touch(); v_num = act_counter();
     dof_structure.clear();
     use_reduction = false;
@@ -446,6 +473,7 @@ namespace getfem {
   void mesh_fem::init_with_mesh(const mesh &me, dim_type Q) {
     GMM_ASSERT1(linked_mesh_ == 0, "Mesh level set already initialized");
     dof_enumeration_made = false;
+    is_uniform_ = false;
     auto_add_elt_pf = 0;
     auto_add_elt_K = dim_type(-1);
     Qdim = Q;
@@ -467,6 +495,7 @@ namespace getfem {
     E_ = mf.E_;
     dof_structure = mf.dof_structure;
     dof_enumeration_made = mf.dof_enumeration_made;
+    is_uniform_ = mf.is_uniform_;
     nb_total_dof = mf.nb_total_dof;
     auto_add_elt_pf = mf.auto_add_elt_pf;
     auto_add_elt_K = mf.auto_add_elt_K;
@@ -491,8 +520,12 @@ namespace getfem {
   mesh_fem::mesh_fem(const mesh &me, dim_type Q)
     { linked_mesh_ = 0; init_with_mesh(me, Q); }
 
-  mesh_fem::mesh_fem(void)
-    { linked_mesh_ = 0; dof_enumeration_made = false; set_qdim(1); }
+  mesh_fem::mesh_fem() {
+    linked_mesh_ = 0;
+    dof_enumeration_made = false;
+    is_uniform_ = true;
+    set_qdim(1);
+  }
 
   mesh_fem::~mesh_fem() { }
 
@@ -538,6 +571,8 @@ namespace getfem {
         } else if (bgeot::casecmp(tmp, "DOF_ENUMERATION") == 0) {
           dal::bit_vector doflst;
           dof_structure.clear(); dof_enumeration_made = false;
+	  is_uniform_ = true;
+	  size_type nbdof_unif = size_type(-1);
           touch(); v_num = act_counter();
           while (true) {
             bgeot::get_token(ist, tmp);
@@ -550,6 +585,11 @@ namespace getfem {
             std::vector<size_type> tab;
             if (convex_index().is_in(ic) && tmp.size() &&
                 isdigit(tmp[0]) && tmp2 == ":") {
+	      size_type nbd = nb_basic_dof_of_element(ic);
+	      if (nbdof_unif == size_type(-1))
+		nbdof_unif = nbd;
+	      else if (nbdof_unif != nbd)
+		is_uniform_ = false;
               tab.resize(nb_basic_dof_of_element(ic));
               for (size_type i=0; i < fem_of_element(ic)->nb_dof(ic);
                    i++) {
@@ -745,23 +785,23 @@ namespace getfem {
   }
 
   struct mf__key_ : public context_dependencies {
-    const mesh *pmesh;
+    const mesh *pmsh;
     dim_type order, qdim;
     mf__key_(const mesh &msh, dim_type o, dim_type q)
-      : pmesh(&msh), order(o), qdim(q)
+      : pmsh(&msh), order(o), qdim(q)
     { add_dependency(msh); }
     bool operator <(const mf__key_ &a) const {
-      if (pmesh < a.pmesh) return true;
-      else if (pmesh > a.pmesh) return false;
+      if (pmsh < a.pmsh) return true;
+      else if (pmsh > a.pmsh) return false;
       else if (order < a.order) return true;
       else if (order > a.order) return false;
       else if (qdim < a.qdim) return true;
       return false;
     }
-    void update_from_context(void) const {}
+    void update_from_context() const {}
     mf__key_(const mf__key_ &mfk) : context_dependencies( ) {
-      pmesh = mfk.pmesh; order = mfk.order; qdim = mfk.qdim;
-      add_dependency(*pmesh);
+      pmsh = mfk.pmsh; order = mfk.order; qdim = mfk.qdim;
+      add_dependency(*pmsh);
     }
   private :
     mf__key_& operator=(const mf__key_ &mfk);
