@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2000-2016 Yves Renard
+ Copyright (C) 2000-2017 Yves Renard
 
  This file is a part of GetFEM++
 
@@ -126,7 +126,7 @@ namespace getfem {
     /* Third invariant det(E) */
     void compute_i3(void) {
       Einv = E;
-      i3_ = gmm::lu_inverse(Einv);
+      i3_ = bgeot::lu_inverse(&(*(Einv.begin())), gmm::mat_nrows(Einv));
       i3_c = true;
     }
 
@@ -287,7 +287,7 @@ namespace getfem {
     scalar_type d;
     do {
       gmm::fill_random(Phi);
-      d = gmm::lu_det(Phi);
+      d = bgeot::lu_det(&(*(Phi.begin())), N);
     } while (d < scalar_type(0.01)); 
     gmm::mult(gmm::transposed(Phi),Phi,E);
     gmm::scale(E,-1.); gmm::add(gmm::identity_matrix(),E); 
@@ -389,16 +389,23 @@ namespace getfem {
   }
   
   scalar_type SaintVenant_Kirchhoff_hyperelastic_law::strain_energy
-  (const base_matrix &E, const base_vector &params, scalar_type) const {
+  (const base_matrix &E, const base_vector &params, scalar_type det_trans) const {
+        // should be optimized, maybe deriving sigma from strain energy
+    if (det_trans <= scalar_type(0))
+    { return 1e200; }
+
     return gmm::sqr(gmm::mat_trace(E)) * params[0] / scalar_type(2)
     + gmm::mat_euclidean_norm_sqr(E) * params[1];
   }
   
   void SaintVenant_Kirchhoff_hyperelastic_law::sigma
-  (const base_matrix &E, base_matrix &result,const base_vector &params, scalar_type) const {
+  (const base_matrix &E, base_matrix &result,const base_vector &params, scalar_type det_trans) const {
     gmm::copy(gmm::identity_matrix(), result);
     gmm::scale(result, params[0] * gmm::mat_trace(E));
     gmm::add(gmm::scaled(E, 2 * params[1]), result);
+    if (det_trans <= scalar_type(0)) {
+      gmm::add(gmm::scaled(E, 1e200), result);
+    }
   }
   void SaintVenant_Kirchhoff_hyperelastic_law::grad_sigma
   (const base_matrix &E, base_tensor &result,const base_vector &params, scalar_type) const {
@@ -479,7 +486,7 @@ namespace getfem {
     // result(0,0,1,0) = 0;
     result(0,0,1,1) = params[1]*params[0]/(1-params[1]*poisonXY);
     result(1,1,0,0) = params[1]*params[0]/(1-params[1]*poisonXY);
-    // result(1,1,0,1) = 0;
+    // result(1,1,0,1) = 0;out
     // result(1,1,1,0) = 0;
     result(1,1,1,1) = params[2]/(1-params[1]*poisonXY);
     // result(0,1,0,0) = 0;
@@ -493,10 +500,10 @@ namespace getfem {
   }
 
   scalar_type Mooney_Rivlin_hyperelastic_law::strain_energy
-  (const base_matrix &E, const base_vector &params,
-   scalar_type /* det_trans*/) const {
-// shouldn't negative det_trans be handled here???
-//    if (compressible && det_trans <= scalar_type(0)) return 1e200;
+  (const base_matrix &E, const base_vector &params
+   ,scalar_type  det_trans) const {
+    //shouldn't negative det_trans be handled here???
+    if (compressible && det_trans <= scalar_type(0)) return 1e200;
     size_type N = gmm::mat_nrows(E);
     GMM_ASSERT1(N == 3, "Mooney Rivlin hyperelastic law only defined "
                 "on dimension 3, sorry");
@@ -504,7 +511,6 @@ namespace getfem {
     gmm::scale(C, scalar_type(2));
     gmm::add(gmm::identity_matrix(), C);
     compute_invariants ci(C);
-
     size_type i=0;
     scalar_type C1 = params[i++]; // C10
     scalar_type W = C1 * (ci.j1() - scalar_type(3));
@@ -521,7 +527,7 @@ namespace getfem {
 
   void Mooney_Rivlin_hyperelastic_law::sigma
   (const base_matrix &E, base_matrix &result,
-   const base_vector &params, scalar_type /*det_trans*/) const {
+   const base_vector &params, scalar_type det_trans) const {
     size_type N = gmm::mat_nrows(E);
     GMM_ASSERT1(N == 3, "Mooney Rivlin hyperelastic law only defined "
                 "on dimension 3, sorry");
@@ -541,9 +547,10 @@ namespace getfem {
       scalar_type D1 = params[i++];
       scalar_type di3 = D1 - D1 / sqrt(gmm::abs(ci.i3()));
       gmm::add(gmm::scaled(ci.grad_i3(), scalar_type(2) * di3), result);
+
 // shouldn't negative det_trans be handled here???
-//      if (det_trans <= scalar_type(0))
-//        gmm::add(gmm::scaled(C, 1e200), result);
+      if (det_trans <= scalar_type(0))
+          gmm::add(gmm::scaled(C, 1e200), result);
     }
   }
 
@@ -595,15 +602,15 @@ namespace getfem {
     nb_params_ = 2;
     if (compressible) ++nb_params_; // D1 != 0
     if (neohookean) --nb_params_;   // C2 == 0
+
   }
 
 
 
 
   scalar_type Neo_Hookean_hyperelastic_law::strain_energy
-  (const base_matrix &E, const base_vector &params,
-   scalar_type det_trans) const {
-    if (det_trans <= scalar_type(0)) return 1e200;
+  (const base_matrix &E, const base_vector &params, scalar_type det_trans) const {
+   if (det_trans <= scalar_type(0)) return 1e200;
     size_type N = gmm::mat_nrows(E);
     GMM_ASSERT1(N == 3, "Neo Hookean hyperelastic law only defined "
                 "on dimension 3, sorry");
@@ -626,7 +633,7 @@ namespace getfem {
 
   void Neo_Hookean_hyperelastic_law::sigma
   (const base_matrix &E, base_matrix &result,
-   const base_vector &params, scalar_type det_trans) const {
+   const base_vector &params , scalar_type det_trans) const {
     size_type N = gmm::mat_nrows(E);
     GMM_ASSERT1(N == 3, "Neo Hookean hyperelastic law only defined "
                 "on dimension 3, sorry");
@@ -644,7 +651,6 @@ namespace getfem {
     else
        gmm::add(gmm::scaled(ci.grad_i3(),
                             lambda/2 - lambda/(2*ci.i3()) - mu / ci.i3()), result);
-    
     if (det_trans <= scalar_type(0))
       gmm::add(gmm::scaled(C, 1e200), result);
   }
@@ -820,7 +826,7 @@ namespace getfem {
     base_matrix C(N, N);
     gmm::copy(gmm::scaled(E, scalar_type(2)), C);
     gmm::add(gmm::identity_matrix(), C);
-    scalar_type det = gmm::lu_det(C);
+    scalar_type det = bgeot::lu_det(&(*(C.begin())), N);
     return a * gmm::mat_trace(C)
       + b * (gmm::sqr(gmm::mat_trace(C)) - 
              gmm::mat_euclidean_norm_sqr(C))/scalar_type(2)
@@ -847,8 +853,8 @@ namespace getfem {
     gmm::add(gmm::scaled(C, -scalar_type(2) * b), result);
     if (det_trans <= scalar_type(0))
       gmm::add(gmm::scaled(C, 1e200), result);
-    else {
-      scalar_type det = gmm::lu_inverse(C);
+          else {
+      scalar_type det = bgeot::lu_inverse(&(*(C.begin())), N);
       gmm::add(gmm::scaled(C, scalar_type(2) * c * det - d), result);
     }
   }
@@ -864,7 +870,7 @@ namespace getfem {
     base_matrix C(N, N);
     gmm::copy(gmm::scaled(E, scalar_type(2)), C);
     gmm::add(gmm::identity_matrix(), C);
-    scalar_type det = gmm::lu_inverse(C);
+    scalar_type det = bgeot::lu_inverse(&(*(C.begin())), N);
     std::fill(result.begin(), result.end(), scalar_type(0));
     for (size_type i = 0; i < N; ++i)
       for (size_type j = 0; j < N; ++j) {
@@ -1103,7 +1109,7 @@ namespace getfem {
         //jyh : end complete graphi
       }
 
-      gmm::scale(sigma, scalar_type(1) / gmm::lu_det(gradphi));
+      gmm::scale(sigma, scalar_type(1) / bgeot::lu_det(&(*(gradphi.begin())), NFem));
 
       if (!tresca) {
         /* von mises: norm(deviator(sigma)) */
@@ -1355,7 +1361,7 @@ namespace getfem {
       size_type N = args[0]->sizes()[0];
       base_matrix M(N, N);
       gmm::copy(args[0]->as_vector(), M.as_vector());
-      scalar_type det = gmm::lu_det(M);
+      scalar_type det = bgeot::lu_det(&(*(M.begin())), N);
       scalar_type tr = scalar_type(0);
       for (size_type i = 0; i < N; ++i) tr += M(i,i);
       if (det > 0)
@@ -1372,7 +1378,7 @@ namespace getfem {
       gmm::copy(args[0]->as_vector(), M.as_vector());
       scalar_type tr = scalar_type(0);
       for (size_type i = 0; i < N; ++i) tr += M(i,i);
-      scalar_type det = gmm::lu_inverse(M);
+      scalar_type det = bgeot::lu_inverse(&(*(M.begin())), N);
       if (det > 0) {
         base_tensor::iterator it = result.begin();
         for (size_type j = 0; j < N; ++j)
@@ -1394,7 +1400,7 @@ namespace getfem {
       gmm::copy(args[0]->as_vector(), M.as_vector());
       scalar_type tr = scalar_type(0);
       for (size_type i = 0; i < N; ++i) tr += M(i,i);
-      scalar_type det = gmm::lu_inverse(M);
+      scalar_type det = bgeot::lu_inverse(&(*(M.begin())), N);
       if (det > 0) {
         base_tensor::iterator it = result.begin();
         for (size_type l = 0; l < N; ++l)
@@ -1434,7 +1440,7 @@ namespace getfem {
         for (size_type j = 0; j < N; ++j)
           tr2 += M(i,j)*M(j,i);
       scalar_type i2 = (tr*tr-tr2)/scalar_type(2);
-      scalar_type det = gmm::lu_det(M);
+      scalar_type det = bgeot::lu_det(&(*(M.begin())), N);
 
       if (det > 0)
         result[0] = i2 / pow(det, scalar_type(2)/scalar_type(3));
@@ -1456,7 +1462,7 @@ namespace getfem {
         for (size_type j = 0; j < N; ++j)
           tr2 += M(i,j)*M(j,i);
       scalar_type i2 = (tr*tr-tr2)/scalar_type(2);
-      scalar_type det = gmm::lu_inverse(M);
+      scalar_type det = bgeot::lu_inverse(&(*(M.begin())), N);
       base_tensor::iterator it = result.begin();
       for (size_type j = 0; j < N; ++j)
         for (size_type i = 0; i < N; ++i, ++it)
@@ -1480,7 +1486,7 @@ namespace getfem {
         for (size_type j = 0; j < N; ++j)
           tr2 += M(i,j)*M(j,i);
       scalar_type i2 = (tr*tr-tr2)/scalar_type(2);
-      scalar_type det = gmm::lu_inverse(M);
+      scalar_type det = bgeot::lu_inverse(&(*(M.begin())), N);
       base_tensor::iterator it = result.begin();
       for (size_type l = 0; l < N; ++l)
         for (size_type k = 0; k < N; ++k)
@@ -1491,7 +1497,7 @@ namespace getfem {
                       - 2.*tr*M(j,i)*((k==l) ? 1. : 0.)/3.
                       + 2.*tr*M(j,i)*M(l,k)/3.
                       - 2.*i2*M(i,k)*M(l,j)/3.
-                      - 2.*((tr*(i==j) ? 1. : 0.)-t[j+N*i]
+                      - 2.*((tr*((i==j) ? 1. : 0.))-t[j+N*i]
                             - 2.*i2*M(j,i)/3)*M(l,k)/3.)
                 / pow(det, scalar_type(2)/scalar_type(3));
     }
@@ -1704,7 +1710,7 @@ namespace getfem {
       gmm::add(gmm::identity_matrix(), F);
       gmm::mult(F, sigma, aux);
       gmm::mult(aux, gmm::transposed(F), sigma);
-      scalar_type det = gmm::lu_det(F);
+      scalar_type det = bgeot::lu_det(&(*(F.begin())), N);
       gmm::scale(sigma, scalar_type(1)/det);
       gmm::copy(sigma.as_vector(), result.as_vector());
     }
@@ -1716,7 +1722,7 @@ namespace getfem {
       base_matrix F(N, N);
       gmm::copy(args[1]->as_vector(), F.as_vector());
       gmm::add(gmm::identity_matrix(), F);
-      scalar_type det = gmm::lu_det(F);
+      scalar_type det = bgeot::lu_det(&(*(F.begin())), N);
 
       base_tensor::iterator it = result.begin();
 
@@ -1738,7 +1744,7 @@ namespace getfem {
           gmm::copy(args[0]->as_vector(), sigma.as_vector());
           gmm::mult(sigma, gmm::transposed(F), aux);
           gmm::mult(F, aux, aux2);
-          gmm::lu_inverse(F);
+          bgeot::lu_inverse(&(*(F.begin())), N);
           for (size_type l = 0; l < N; ++l)
             for (size_type k = 0; k < N; ++k)
               for (size_type j = 0; j < N; ++j)
@@ -1783,7 +1789,7 @@ namespace getfem {
       gmm::add(Gu, E); gmm::add(gmm::transposed(Gu), E);
       gmm::scale(E, scalar_type(0.5));
       gmm::add(gmm::identity_matrix(), Gu);
-      scalar_type det = gmm::lu_det(Gu);
+      scalar_type det = bgeot::lu_det(&(*(Gu.begin())), N);
 
       AHL->sigma(E, sigma, params, det);
       gmm::copy(sigma.as_vector(), result.as_vector());
@@ -1802,7 +1808,7 @@ namespace getfem {
       gmm::add(Gu, E); gmm::add(gmm::transposed(Gu), E);
       gmm::scale(E, scalar_type(0.5));
       gmm::add(gmm::identity_matrix(), Gu);
-      scalar_type det = gmm::lu_det(Gu);
+      scalar_type det = bgeot::lu_det(&(*(Gu.begin())), N);
 
       GMM_ASSERT1(nder == 1, "Sorry, the derivative of this hyperelastic "
                   "law with respect to its parameters is not available.");
@@ -1854,7 +1860,7 @@ namespace getfem {
       gmm::add(Gu, E); gmm::add(gmm::transposed(Gu), E);
       gmm::scale(E, scalar_type(0.5));
       gmm::add(gmm::identity_matrix(), Gu);
-      scalar_type det = gmm::lu_det(Gu); // not necessary here
+      scalar_type det = bgeot::lu_det(&(*(Gu.begin())), N); // not necessary here
 
       result[0] = AHL->strain_energy(E, params, det);
     }
@@ -1871,7 +1877,7 @@ namespace getfem {
       gmm::add(Gu, E); gmm::add(gmm::transposed(Gu), E);
       gmm::scale(E, scalar_type(0.5));
       gmm::add(gmm::identity_matrix(), Gu);
-      scalar_type det = gmm::lu_det(Gu); // not necessary here
+      scalar_type det = bgeot::lu_det(&(*(Gu.begin())), N); // not necessary here
 
       GMM_ASSERT1(nder == 1, "Sorry, Cannot derive the potential with "
                   "respect to law parameters.");
@@ -1896,7 +1902,7 @@ namespace getfem {
       gmm::add(Gu, E); gmm::add(gmm::transposed(Gu), E);
       gmm::scale(E, scalar_type(0.5));
       gmm::add(gmm::identity_matrix(), Gu);
-      scalar_type det = gmm::lu_det(Gu);
+      scalar_type det = bgeot::lu_det(&(*(Gu.begin())), N);
 
       GMM_ASSERT1(nder1 == 1 && nder2 == 1, "Sorry, Cannot derive the "
                   "potential with respect to law parameters.");
