@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2002-2016 Konstantinos Poulios, Amandine Cottaz, Yves Renard
+ Copyright (C) 2002-2017 Konstantinos Poulios, Amandine Cottaz, Yves Renard
 
  This file is a part of GetFEM++
 
@@ -326,6 +326,73 @@ namespace getfem {
   };
 
 
+  // Ball Projection operator.
+  struct Ball_projection_operator : public ga_nonlinear_operator {
+    bool result_size(const arg_list &args, bgeot::multi_index &sizes) const {
+      if (args.size() != 2 || args[0]->sizes().size() > 2
+          || args[0]->sizes().size() < 1 || args[1]->size() != 1) return false;
+      if (args[0]->sizes().size() == 1)
+        ga_init_vector(sizes, args[0]->sizes()[0]);
+      else
+        ga_init_matrix(sizes, args[0]->sizes()[0], args[0]->sizes()[1]);
+      return true;
+    }
+
+    // Value : ru/|u| if |u| > r, else u 
+    void value(const arg_list &args, base_tensor &result) const {
+      const base_tensor &t = *args[0];
+      scalar_type r = (*args[1])[0];
+      scalar_type no = gmm::vect_norm2(t.as_vector());
+      if (no > r)
+	gmm::copy(gmm::scaled(t.as_vector(), r/no), result.as_vector());
+      else
+	gmm::copy(t.as_vector(), result.as_vector());
+    }
+
+    // Derivative
+    void derivative(const arg_list &args, size_type n,
+                    base_tensor &result) const {
+      const base_tensor &t = *args[0];
+      size_type N = t.size();
+      scalar_type r = (*args[1])[0];
+      scalar_type no = gmm::vect_norm2(t.as_vector()), rno3 = r/(no*no*no);
+
+      gmm::clear(result.as_vector());
+
+      switch(n) {
+
+      case 1 : // derivative with respect to u
+	if (r > 0) {
+	  if (no <= r) {
+	    for (size_type i = 0; i < N; ++i)
+	      result[i*N+i] += scalar_type(1);
+	  } else {
+	    for (size_type i = 0; i < N; ++i) {
+	      result[i*N+i] += r/no;
+	      for (size_type j = 0; j < N; ++j)
+		result[j*N+i] -= t[i]*t[j]*rno3;
+	    }
+	  }
+	}
+	break;
+      case 2 : // derivative with respect to r
+	if (r > 0 && no > r) {
+	  for (size_type i = 0; i < N; ++i)
+	    result[i] = t[i]/no;
+	}
+	break;
+      default : GMM_ASSERT1(false, "Wrong derivative number");
+      }
+    }
+    
+    // Second derivative : not implemented
+    void second_derivative(const arg_list &/*args*/, size_type, size_type,
+                           base_tensor &/*result*/) const {
+      GMM_ASSERT1(false, "Sorry, second derivative not implemented");
+    }
+  };
+
+
   // Normalized_reg vector/matrix operator : Regularized Vector/matrix divided by its Frobenius norm
   struct normalized_reg_operator : public ga_nonlinear_operator {
     bool result_size(const arg_list &args, bgeot::multi_index &sizes) const {
@@ -484,6 +551,8 @@ namespace getfem {
                                 std::make_shared<normalized_operator>());
     PREDEF_OPERATORS.add_method("Normalized_reg",
                                 std::make_shared<normalized_reg_operator>());
+    PREDEF_OPERATORS.add_method("Ball_projection",
+                                std::make_shared<Ball_projection_operator>());
     PREDEF_OPERATORS.add_method("Von_Mises_projection",
                                 std::make_shared<Von_Mises_projection_operator>());
     return true;
@@ -1111,7 +1180,7 @@ namespace getfem {
     bool hardening = (lawname.find("_hardening") != std::string::npos);
     size_type nhard = hardening ? 2 : 0;
 
-    GMM_ASSERT1(varnames.size() == hardening ? 4 : 3,
+    GMM_ASSERT1(varnames.size() == (hardening ? 4 : 3),
                 "Incorrect number of variables: " << varnames.size());
     GMM_ASSERT1(params.size() >= 3+nhard &&
                 params.size() <= 5+nhard,
@@ -1853,7 +1922,7 @@ namespace getfem {
                                              gmm::sub_interval(0,mf_sigma.nb_dof())),
                              Sigma_n);
 
-      if (pmf_data != NULL) {
+      if (pmf_data) {
         gmm::resize(mu, pmf_data->nb_basic_dof());
         gmm::resize(lambda, pmf_data->nb_basic_dof());
         gmm::resize(threshold, pmf_data->nb_basic_dof());
@@ -1978,7 +2047,7 @@ namespace getfem {
      const mesh_im &mim,
      const mesh_fem &mf_u,
      const mesh_fem &mf_sigma,
-     const mesh_fem &mf_data,
+     const mesh_fem *pmf_data,
      const model_real_plain_vector &u_n,
      const model_real_plain_vector &u_np1,
      const model_real_plain_vector &sigma_n,
@@ -1991,14 +2060,14 @@ namespace getfem {
     GMM_ASSERT1(mf_u.get_qdim() == mf_u.linked_mesh().dim(),
                 "wrong qdim for the mesh_fem");
 
-    elastoplasticity_nonlinear_term gradplast(mim, mf_u, mf_sigma, &mf_data,
+    elastoplasticity_nonlinear_term gradplast(mim, mf_u, mf_sigma, pmf_data,
                                               u_n, u_np1, sigma_n,
                                               threshold, lambda, mu,
                                               t_proj, GRADPROJ, false);
 
     generic_assembly assem;
 
-    if (&(mf_data)!=NULL)
+    if (pmf_data)
       assem.set("lambda=data$1(#3); mu=data$2(#3);"
                 "t=comp(NonLin(#2).vGrad(#1).vGrad(#1).Base(#3))(i,j,:,:,:,:,:,:,i,j,:);"
                 "M(#1,#1)+=  sym(t(k,l,:,l,k,:,m).mu(m)+t(k,l,:,k,l,:,m).mu(m)+t(k,k,:,l,l,:,m).lambda(m))");
@@ -2010,8 +2079,8 @@ namespace getfem {
     assem.push_mi(mim);
     assem.push_mf(mf_u);
     assem.push_mf(mf_sigma);
-    if (&(mf_data)!=NULL)
-      assem.push_mf(mf_data);
+    if (pmf_data)
+      assem.push_mf(*pmf_data);
     assem.push_data(lambda);
     assem.push_data(mu);
     assem.push_nonlinear_term(&gradplast);
@@ -2077,7 +2146,7 @@ namespace getfem {
       if (version & model::BUILD_MATRIX) {
         gmm::clear(matl[0]);
         asm_elastoplasticity_tangent_matrix
-          (matl[0], mim, mf_u, mf_sigma, *mf_data, u_n,
+          (matl[0], mim, mf_u, mf_sigma, mf_data, u_n,
            u_np1, sigma_n, lambda, mu, threshold, *t_proj, rg);
       }
 
@@ -2307,4 +2376,3 @@ namespace getfem {
 
 
 }  /* end of namespace getfem.  */
-
