@@ -1,6 +1,6 @@
 /*===========================================================================
 
- Copyright (C) 2012-2016 Yves Renard, Konstantinos Poulios
+ Copyright (C) 2012-2017 Yves Renard, Konstantinos Poulios
 
  This file is a part of GetFEM++
 
@@ -765,7 +765,7 @@ namespace getfem {
 
   void projected_fem::gauss_pts_stats(unsigned &ming, unsigned &maxg,
                                       scalar_type &meang) const {
-    std::vector<unsigned> v(mf_source.linked_mesh().convex_index().last_true()+1);
+    std::vector<unsigned> v(mf_source.linked_mesh().nb_allocated_convex());
     std::map<size_type,elt_projection_data>::const_iterator eit;
     for (eit = elements.begin(); eit != elements.end(); ++eit) {
       std::map<size_type,gausspt_projection_data>::const_iterator git;
@@ -815,7 +815,7 @@ namespace getfem {
       store_values(store_val), blocked_dofs(blocked_dofs_), mi2(2), mi3(3) {
     this->add_dependency(mf_source);
     this->add_dependency(mim_target);
-    is_pol = is_lag = false; es_degree = 5;
+    is_pol = is_lag = is_standard_fem = false; es_degree = 5;
     is_equiv = real_element_defined = true;
     ntarget_dim = mf_source.get_qdim();
 
